@@ -10,6 +10,9 @@
 //#define PG_DEBUGRACE	1
 
 //#define PG_STATS	1	    // Collect extra usage statistics
+#define PG_DEBUGHISTORY	     0	    // If >0, it is the number of records
+				    // we keep history on the connection
+				    // with the database
 #define PG_DEADLOCK_SENTINEL 0	    // If >0, defines the number seconds
 				    // a lock can be held before the deadlock
 				    // report is being dumped to stderr
