@@ -111,6 +111,9 @@
 /* Define this if your ld uses -rpath, but your cc wants -Wl,-rpath, */
 #undef USE_Wl
 
+/* Define this if your ld uses Darwin-style -rpath, but your cc wants -Wl,-rpath, */
+#undef USE_Wl_rpath_darwin
+
 /* Define this if your ld uses -R, but your cc wants -Wl,-R */
 #undef USE_Wl_R
 
@@ -153,12 +156,25 @@
  * __chkstk. */
 #undef HAVE_BROKEN_CHKSTK
 
+/* Define if you have a working getcwd(3) (ie one that returns a malloc()ed
+ * buffer if the first argument is NULL).
+ *
+ * Define to 1 if the second argument being 0 causes getcwd(3) to allocate
+ * a buffer of suitable size (ie never fail with ERANGE).
+ *
+ * Define to 0 if the second argument MUST be > 0.
+ */
+#undef HAVE_WORKING_GETCWD
+
 /* Define for solaris */
 #undef SOLARIS
 
 /* Define if the closedir function returns void instead of int.  */
 #undef VOID_CLOSEDIR
 
+/* Number of args to mkdir() */
+#define MKDIR_ARGS 2
+
 /* Define to 'int' if <sys/time.h> doesn't */
 #undef time_t
 
