@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: d0533bae20ed6798d5747f5e710f214b1b979c00 $
 */
 
 #ifndef ARRAY_H
@@ -94,6 +93,12 @@ extern struct array *gc_internal_array;
 #define allocate_array(X) low_allocate_array((X),0)
 #define allocate_array_no_init(X,Y) low_allocate_array((X),(Y))
 
+/* Special value used for cmpfuns to signify that two values aren't
+ * equal and have no order relation, i.e. one is neither greater nor
+ * less than the other. It consists of the largest bit (under the sign
+ * bit) set. */
+#define CMPFUN_UNORDERED (INT_MAX - (INT_MAX >> 1))
+
 typedef int (*cmpfun)(const struct svalue *, const struct svalue *);
 typedef int (*short_cmpfun)(union anything *, union anything *);
 typedef short_cmpfun (*cmpfun_getter)(TYPE_T);
@@ -107,6 +112,7 @@ typedef short_cmpfun (*cmpfun_getter)(TYPE_T);
 PMOD_EXPORT struct array *real_allocate_array(ptrdiff_t size, ptrdiff_t extra_space);
 PMOD_EXPORT void really_free_array(struct array *v);
 PMOD_EXPORT void do_free_array(struct array *a);
+PMOD_EXPORT void clear_array(struct array *a);
 PMOD_EXPORT struct array *array_set_flags(struct array *a, int flags);
 PMOD_EXPORT void array_index(struct svalue *s,struct array *v,INT32 ind);
 PMOD_EXPORT struct array *array_column (struct array *data, struct svalue *index,
@@ -158,10 +164,6 @@ PMOD_EXPORT struct array *add_arrays(struct svalue *argp, INT32 args);
 PMOD_EXPORT int array_equal_p(struct array *a, struct array *b, struct processing *p);
 PMOD_EXPORT struct array *merge_array_with_order(struct array *a,
 						 struct array *b, INT32 op);
-PMOD_EXPORT struct array *merge_array_without_order2(struct array *a, struct array *b,INT32 op);
-PMOD_EXPORT struct array *merge_array_without_order(struct array *a,
-					struct array *b,
-					INT32 op);
 PMOD_EXPORT struct array *subtract_arrays(struct array *a, struct array *b);
 PMOD_EXPORT struct array *and_arrays(struct array *a, struct array *b);
 int array_is_constant(struct array *a,
@@ -193,7 +195,7 @@ void array_replace(struct array *a,
 PMOD_EXPORT void check_array(struct array *a);
 void check_all_arrays(void);
 #endif
-PMOD_EXPORT void visit_array (struct array *a, int action);
+PMOD_EXPORT void visit_array (struct array *a, int action, void *extra);
 void gc_mark_array_as_referenced(struct array *a);
 void real_gc_cycle_check_array(struct array *a, int weak);
 unsigned gc_touch_all_arrays(void);
@@ -209,13 +211,17 @@ void debug_dump_array(struct array *a);
 void count_memory_in_arrays(size_t *num_, size_t *size_);
 PMOD_EXPORT struct array *explode_array(struct array *a, struct array *b);
 PMOD_EXPORT struct array *implode_array(struct array *a, struct array *b);
+
+/* Automap internals. */
+void assign_array_level_value( struct array *a, struct svalue *b, int level );
+void assign_array_level( struct array *a, struct array *b, int level );
 /* Prototypes end here */
 
 #define array_get_flags(a) ((a)->flags)
 
-#define visit_array_ref(A, REF_TYPE)					\
+#define visit_array_ref(A, REF_TYPE, EXTRA)				\
   visit_ref (pass_array (A), (REF_TYPE),				\
-	     (visit_thing_fn *) &visit_array, NULL)
+	     (visit_thing_fn *) &visit_array, (EXTRA))
 #define gc_cycle_check_array(X, WEAK) \
   gc_cycle_enqueue((gc_cycle_check_cb *) real_gc_cycle_check_array, (X), (WEAK))
 
@@ -253,8 +259,8 @@ PMOD_EXPORT struct array *implode_array(struct array *a, struct array *b);
       } END_ACCEPT_UNFINISHED_TYPE_FIELDS;				\
       /* Unless the user does something, the type field will contain */	\
       /* BIT_MIXED|BIT_UNFINISHED from the allocation above. */		\
-      MEMCPY((char *) (ITEM(base_sval[-1].u.array) + oldsize__),	\
-	     (char *) base_sval, diff__ * sizeof(struct svalue));	\
+      memcpy(ITEM(base_sval[-1].u.array) + oldsize__,                   \
+	     base_sval, diff__ * sizeof(struct svalue));                \
       Pike_sp = base_sval;						\
     }									\
   } while (0)
@@ -271,7 +277,7 @@ PMOD_EXPORT struct array *implode_array(struct array *a, struct array *b);
       AGGR_ARR_CHECK (base_sval, 0);					\
     if (base_sval[-1].u.array->type_field & BIT_UNFINISHED)		\
       array_fix_type_field(Pike_sp[-1].u.array);			\
-    DO_IF_DEBUG(if (Pike_sp[-1].type != T_ARRAY) {			\
+    DO_IF_DEBUG(if (TYPEOF(Pike_sp[-1]) != T_ARRAY) {			\
 	Pike_fatal("Lost track of aggregated array.\n");		\
       });								\
   } while (0)
@@ -322,7 +328,7 @@ PMOD_EXPORT struct array *implode_array(struct array *a, struct array *b);
 									 \
   check_destructed(s_);							 \
 									 \
-  v_->type_field |= 1 << s_->type;					 \
+  v_->type_field |= 1 << TYPEOF(*s_);					 \
   assign_svalue_no_free( ITEM(v_) + index_, s_);			 \
 }while(0)
 #define array_set_index(V,I,S) do {					\
@@ -337,7 +343,7 @@ PMOD_EXPORT struct array *implode_array(struct array *a, struct array *b);
 									 \
   check_destructed(s_);							 \
 									 \
-  v_->type_field |= 1 << s_->type;					 \
+  v_->type_field |= 1 << TYPEOF(*s_);					 \
   assign_svalue( ITEM(v_) + index_, s_);				\
 }while(0)
 
