@@ -1982,6 +1982,7 @@ struct pike_frame *alloc_pike_frame(void)
       res->flags=0;
       res->next=0;
       res->scope=0;
+      res->pc = NULL;
 
       DO_IF_SECURITY(
         if(CURRENT_CREDS) {
@@ -3053,9 +3054,10 @@ void low_safe_apply_handler(const char *fun,
 
 PMOD_EXPORT void push_text( const char *x )
 {
+    struct pike_string *s = make_shared_string(x);
     struct svalue *_sp_ = Pike_sp++;
     SET_SVAL_SUBTYPE(*_sp_, 0);
-    _sp_->u.string=make_shared_string(x);
+    _sp_->u.string=s;
     debug_malloc_touch(_sp_->u.string);
     SET_SVAL_TYPE(*_sp_, PIKE_T_STRING);
 }
