@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: b3aa0721a8ec2dccc331f8aca0eddbf379fdc518 $
 */
 
 /*
@@ -22,8 +21,8 @@
 
 /* Generic */
 #define GOBBLE(c) (LOOK()==c?(SKIP(),1):0)
-#define SKIPSPACE() do { while(ISSPACE(LOOK()) && LOOK()!='\n') SKIP(); }while(0)
-#define SKIPWHITE() do { while(ISSPACE(LOOK())) SKIP(); }while(0)
+#define SKIPSPACE() do { while(isspace(LOOK()) && LOOK()!='\n') SKIP(); }while(0)
+#define SKIPWHITE() do { while(isspace(LOOK())) SKIP(); }while(0)
 #define SKIPUPTO(X) do { while(LOOK()!=(X) && LOOK()) SKIP(); }while(0)
 
 #if (SHIFT == 0)
@@ -44,7 +43,7 @@
 } while(0)
 
 #define TWO_CHAR(X,Y) ((X)<<8)+(Y)
-#define ISWORD(X) ((len == strlen(X)) && !MEMCMP(buf,X,strlen(X)))
+#define ISWORD(X) ((len == strlen(X)) && !memcmp(buf,X,len))
 
 /*
  * Function renaming
@@ -54,7 +53,7 @@
 #define yylex yylex0
 #define low_yylex low_yylex0
 #define lex_atoi atoi
-#define lex_strtol STRTOL
+#define lex_strtol strtol
 #define lex_strtod my_strtod
 #define lex_isidchar isidchar
 
@@ -62,7 +61,7 @@
 
 #define LOOK() INDEX_CHARP(lex->pos,0,SHIFT)
 #define SKIP() (lex->pos += (1<<SHIFT))
-#define SKIPN(N) (lex->pos += ((N)<<SHIFT))
+#define SKIPN(N) (lex->pos += (N) < 0 ? -(-(N) << SHIFT) : ((N)<<SHIFT))
 #define GETC() (SKIP(),INDEX_CHARP(lex->pos-(1<<SHIFT),0,SHIFT))
 
 #define READBUF(X) do {				\
@@ -75,7 +74,7 @@
 
 #define TWO_CHAR(X,Y) ((X)<<8)+(Y)
 
-#define ISWORD(X) ((len == strlen(X)) && low_isword(buf, X, strlen(X)))
+#define ISWORD(X) ((len == strlen(X)) && low_isword(buf, X, len))
 
 #if (SHIFT == 1)
 
@@ -215,7 +214,7 @@ int parse_esc_seq (WCHAR *buf, p_wchar2 *chr, ptrdiff_t *len)
     case '0': case '1': case '2': case '3':
     case '4': case '5': case '6': case '7': {
       unsigned INT32 n = c-'0';
-      for (l = 1; buf[l] >= '0' && buf[l] <= '8'; l++) {
+      for (l = 1; buf[l] >= '0' && buf[l] < '8'; l++) {
 	if (DO_UINT32_MUL_OVERFLOW(n, 8, &n))
 	  of = 1;
 	else
@@ -228,6 +227,11 @@ int parse_esc_seq (WCHAR *buf, p_wchar2 *chr, ptrdiff_t *len)
       c = (p_wchar2)n;
       break;
     }
+
+    case '8': case '9':
+      if( Pike_compiler->compiler_pass == 1 )
+	yywarning("%c is not a valid octal digit.", c);
+      break;
       
     case 'x': {
       unsigned INT32 n=0;
@@ -288,6 +292,7 @@ int parse_esc_seq (WCHAR *buf, p_wchar2 *chr, ptrdiff_t *len)
     case 'U': {
       /* FIXME: Do we need compat goo to turn this off? */
       /* Note: Code dup in gobble_identifier in preprocessor.h. */
+      unsigned INT32 n = 0;
       int stop, longq;
       l = 1;
       if (buf[1] == c) {
@@ -308,23 +313,23 @@ int parse_esc_seq (WCHAR *buf, p_wchar2 *chr, ptrdiff_t *len)
 	stop = l + 8;
 	longq = 1;
       }
-      c = 0;
       for (; l < stop; l++)
 	switch (buf[l]) {
 	  case '0': case '1': case '2': case '3': case '4':
 	  case '5': case '6': case '7': case '8': case '9':
-	    c = 16 * c + buf[l] - '0';
+	    n = 16 * n + buf[l] - '0';
 	    break;
 	  case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
-	    c = 16 * c + buf[l] - 'a' + 10;
+	    n = 16 * n + buf[l] - 'a' + 10;
 	    break;
 	  case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
-	    c = 16 * c + buf[l] - 'A' + 10;
+	    n = 16 * n + buf[l] - 'A' + 10;
 	    break;
 	  default:
 	    *len = l;
 	    return longq ? 8 : 7;
 	}
+      c = (p_wchar2)n;
     }
   }
 
@@ -353,16 +358,16 @@ static p_wchar2 char_const(struct lex *lex)
       return 0;
     case 4: case 5: case 6:
       if( Pike_compiler->compiler_pass == 1 )
-        yywarning ("Too large character value in escape.");
+        yyerror ("Too large character value in escape.");
       c = -1;
       break;
     case 7:
       if( Pike_compiler->compiler_pass == 1 )
-        yywarning ("Too few hex digits in \\u escape.");
+        yyerror ("Too few hex digits in \\u escape.");
       return '\\';
     case 8:
       if( Pike_compiler->compiler_pass == 1 )
-        yywarning ("Too few hex digits in \\U escape.");
+        yyerror ("Too few hex digits in \\U escape.");
       return '\\';
   }
   SKIPN (l);
@@ -425,7 +430,9 @@ static struct pike_string *readstring(struct lex *lex)
 
 
 
+#if LEXDEBUG>4
 static int low_yylex(struct lex *lex, YYSTYPE *);
+#endif /* LEXDEBUG>4 */
 int yylex(struct lex *lex, YYSTYPE *yylval)
 #if LEXDEBUG>4
 {
@@ -452,7 +459,7 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
   char *buf;
 
 #ifdef __CHECKER__
-  MEMSET((char *)yylval,0,sizeof(YYSTYPE));
+  memset(yylval,0,sizeof(YYSTYPE));
 #endif
 #ifdef MALLOC_DEBUG
   check_sfltable();
@@ -480,6 +487,14 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
 	case TWO_CHAR('a','r'):
 	  if(ISWORD("array")) return TOK_ARRAY_ID;
 	  break;
+	case TWO_CHAR('a','u'):
+	  if(ISWORD("auto")) {
+	    if (Pike_compiler->compiler_pass == 1) {
+	      yywarning("auto will soon be a reserved keyword.");
+	    }
+	    break;
+	  }
+	  break;
 	case TWO_CHAR('b','r'):
 	  if(ISWORD("break")) return TOK_BREAK;
 	  break;
@@ -493,6 +508,12 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
 	case TWO_CHAR('c','o'):
 	  if(ISWORD("constant")) return TOK_CONSTANT;
 	  if(ISWORD("continue")) return TOK_CONTINUE;
+	  if(ISWORD("const")) {
+	    if (Pike_compiler->compiler_pass == 1) {
+	      yywarning("const will soon be a reserved keyword.");
+	    }
+	    break;
+	  }
 	  break;
 	case TWO_CHAR('d','e'):
 	  if(ISWORD("default")) return TOK_DEFAULT;
@@ -509,12 +530,6 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
 	case TWO_CHAR('e','x'):
 	  if(ISWORD("extern")) return TOK_EXTERN;
 	  break;
-#ifdef WITH_FACETS
-#warning "facet" keyword needs compat if enabled by default
-	case TWO_CHAR('f','a'):
-	  if(ISWORD("facet")) return TOK_FACET;
-	  break;
-#endif
 	case TWO_CHAR('f','i'):
 	  if(ISWORD("final")) return TOK_FINAL_ID;
 	  break;
@@ -532,7 +547,7 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
 	  if(ISWORD("gauge")) return TOK_GAUGE;
 	  break;
 	case TWO_CHAR('g','l'):
-	  if (ISWORD("global") && !TEST_COMPAT(7,2)) return TOK_GLOBAL;
+	  if (ISWORD("global")) return TOK_GLOBAL;
 	  break;
 	case TWO_CHAR('i','f'):
 	  if(ISWORD("if")) return TOK_IF;
@@ -715,9 +730,19 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
 
       switch(len>0?INDEX_CHARP(buf, 0, SHIFT):0)
       {
+      case 'l':
+	if (ISWORD("line"))
+	{
+	  SKIPSPACE();
+
+	  if (LOOK() < '0' || LOOK() > '9') goto unknown_directive;
+
+	  READBUF(C!='\n' && C!=' ' && C!='\t');
+	  /* fallthrough */
+	} else goto unknown_directive;
       case '0': case '1': case '2': case '3': case '4':
       case '5': case '6': case '7': case '8': case '9':
-	lex->current_line=lex_atoi(buf)-1;
+	lex->current_line = lex_strtol(buf, NULL, 10)-1;
 	SKIPSPACE();
 	if(GOBBLE('"'))
 	{
@@ -803,30 +828,18 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
 	/* FALL_THROUGH */
 
       default:
-	/* FIXME: This doesn't look all that safe...
-	 * buf isn't NUL-terminated, and it won't work on wide strings.
-	 * /grubba 1999-02-20
-	 * It also modified a shared string... ie. this code was not
-	 * good at all. Fixed.
-	 * /grubba 2000-11-19 (in Versailles)
-	 */
+unknown_directive:
 	if (len < 256) {
 #if (SHIFT == 0)
 	  struct pike_string *dir =
 	    make_shared_binary_string(buf, len);
-#else /* SHIFT != 0 */
-#if (SHIFT == 1)
+#elif (SHIFT == 1)
 	  struct pike_string *dir =
 	    make_shared_binary_string1((p_wchar1 *)buf, len);
-#else /* SHIFT != 1 */
-#if (SHIFT == 2)
+#elif (SHIFT == 2)
 	  struct pike_string *dir =
 	    make_shared_binary_string2((p_wchar2 *)buf, len);
-#else /* SHIFT != 2 */
-#error Unsupported SHIFT.
-#endif /* SHIFT == 2 */
-#endif /* SHIFT == 1 */
-#endif /* SHIFT == 0 */
+#endif
 	  my_yyerror("Unknown preprocessor directive %S.", dir);
 	  free_string(dir);
 	} else {
@@ -842,27 +855,65 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
       continue;
 
     case '\'':
-      switch(c=GETC())
       {
-      case 0:
-	lex->pos -= (1<<SHIFT);
-	yyerror("Unexpected end of file\n");
-	break;
+        unsigned int l = 0;
+        struct svalue res = svalue_int_zero;
+        MP_INT bigint;
 
-      case '\\':
-	c = char_const(lex);
-	break;
+        while(1)
+        {
+          INT32 tmp;
+          switch( (tmp=GETC()) )
+          {
+            case 0:
+              lex->pos -= (1<<SHIFT);
+              yyerror("Unexpected end of file\n");
+              goto return_char;
 
-      case '\'':
-	yyerror("Zero-length character constant.");
-	debug_malloc_pass( yylval->n=mkintnode(0) );
-	return TOK_NUMBER;
+            case '\\':
+              tmp = char_const(lex);
+              /* fallthrough. */
+            default:
+              l++;
+              if( l == sizeof(INT_TYPE)-1 )
+              {
+                /* overflow possible. Switch to bignums. */
+                mpz_init(&bigint);
+                mpz_set_ui(&bigint,res.u.integer);
+                TYPEOF(res) = PIKE_T_OBJECT;
+              }
+
+              if( l >= sizeof(INT_TYPE)-1 )
+              {
+                mpz_mul_2exp(&bigint,&bigint,8);
+                mpz_add_ui(&bigint,&bigint,tmp);
+              }
+              else
+              {
+                res.u.integer <<= 8;
+                res.u.integer |= tmp;
+              }
+              break;
+
+            case '\'':
+              if( l == 0 )
+                yyerror("Zero-length character constant.");
+              goto return_char;
+          }
+        }
+      return_char:
+        if( TYPEOF(res) == PIKE_T_OBJECT )
+        {
+          push_bignum( &bigint );
+          mpz_clear(&bigint);
+          reduce_stack_top_bignum();
+          res = *--Pike_sp;
+        }
+        debug_malloc_pass( yylval->n=mksvaluenode(&res) );
+        free_svalue( &res );
+        return TOK_NUMBER;
       }
-      if(!GOBBLE('\''))
-	yyerror("Unterminated character constant.");
-      debug_malloc_pass( yylval->n=mkintnode(c) );
-      return TOK_NUMBER;
-	
+      /* notreached. */
     case '"':
     {
       struct pike_string *s=readstring(lex);
@@ -870,7 +921,7 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
       free_string(s);
       return TOK_STRING;
     }
-  
+
     case ':':
       if(GOBBLE(':')) return TOK_COLON_COLON;
       return c;
@@ -892,7 +943,7 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
     case '0':
     {
       int base = 0;
-      
+
       if(GOBBLE('b') || GOBBLE('B'))
       {
 	base = 2;
@@ -903,9 +954,7 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
 	struct svalue sval;
 	base = 16;
       read_based_number:
-	sval.type = PIKE_T_INT;
-	sval.subtype = NUMBER_NUMBER;
-	sval.u.integer = 0;
+	SET_SVAL(sval, PIKE_T_INT, NUMBER_NUMBER, integer, 0);
 	safe_wide_string_to_svalue_inumber(&sval,
 					   lex->pos,
 					   &lex->pos,
@@ -918,13 +967,13 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
 	return TOK_NUMBER;
       }
     }
-  
+
     case '1': case '2': case '3': case '4':
     case '5': case '6': case '7': case '8': case '9':
     {
       char *p1, *p2;
       double f;
-      long l = 0;		/* GCC thinks l is unitialized here. Hmm..? */
+      long l;
       struct svalue sval;
 
       lex->pos -= (1<<SHIFT);
@@ -938,9 +987,7 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
     read_float:
       f=lex_strtod(lex->pos, &p1);
 
-      sval.type = PIKE_T_INT;
-      sval.subtype = NUMBER_NUMBER;
-      sval.u.integer = 0;      
+      SET_SVAL(sval, PIKE_T_INT, NUMBER_NUMBER, integer, 0);
 
       safe_wide_string_to_svalue_inumber(&sval,
 					 lex->pos,
@@ -951,7 +998,7 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
       if(p1>p2)
       {
 	/* Floating point or version. */
-	if ((sval.type == PIKE_T_INT) &&
+	if ((TYPEOF(sval) == PIKE_T_INT) &&
 	    (INDEX_CHARP(p2, 0, SHIFT) == '.')) {
 	  int major = sval.u.integer;
 	  char *p3 = p2;
@@ -966,8 +1013,8 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
 					     0,
 					     SHIFT);
 	  dmalloc_touch_svalue(&sval);
-	  if ((sval.type == PIKE_T_INT) && (p3 > p2)) {
-	    for (l=0; ISSPACE(INDEX_CHARP(p3, l, SHIFT)); l++)
+	  if ((TYPEOF(sval) == PIKE_T_INT) && (p3 > p2)) {
+	    for (l=0; isspace(INDEX_CHARP(p3, l, SHIFT)); l++)
 	      ;
 	    if ((INDEX_CHARP(p3, l, SHIFT) == ':') &&
 		(INDEX_CHARP(p3, l+1, SHIFT) == ':')) {
@@ -986,8 +1033,7 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
 #endif /* 0 */
 	lex->pos=p1;
 	if (lex_isidchar (LOOK())) {
-	  my_yyerror ("Invalid char '%c' in constant.",
-		      INDEX_CHARP (lex->pos, l, SHIFT));
+	  my_yyerror ("Invalid char '%c' in constant.", LOOK());
 	  do SKIP(); while (lex_isidchar (LOOK()));
 	}
 	return TOK_FLOAT;
@@ -997,9 +1043,17 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
 	free_svalue(&sval);
 	debug_malloc_touch(yylval->n);
 	lex->pos=p2;
-	if (lex_isidchar (LOOK())) {
-	  my_yyerror ("Invalid char '%c' in constant.",
-		      INDEX_CHARP (lex->pos, l, SHIFT));
+	if (lex_isidchar (LOOK()))
+        {
+          if( GOBBLE('b') )
+            if( GOBBLE( 'i' ) )
+              if( GOBBLE( 't' ) )
+              {
+                GOBBLE('s');
+                return TOK_BITS;
+              }
+
+	  my_yyerror ("Invalid char '%c' in constant.", LOOK());
 	  do SKIP(); while (lex_isidchar (LOOK()));
 	}
 	return TOK_NUMBER;
@@ -1074,17 +1128,43 @@ static int low_yylex(struct lex *lex, YYSTYPE *yylval)
       if(GOBBLE('<')) return TOK_MULTISET_START;
       return '(';
 
-    case ']':
     case '?':
+      if(GOBBLE(':'))
+        return TOK_LOR;
+
+      if(GOBBLE('-') ) /* safe index: ?->  or ?[] */
+      {
+        if( GOBBLE( '>' ) ) /* ?-> */
+            return TOK_SAFE_INDEX;
+        SKIPN(-1); /* Undo GOBBLE('-') above */
+      }
+
+      /* Probably wanted:
+
+         ?.   for safe constant index
+         ?[]  for safe [] index
+
+         They however conflict with valid ?: syntaxes.
+      */
+
+      /* if( GOBBLE('.' ) ) */
+      /*   return TOK_SAFE_INDEX; */
+
+    case ']':
     case ',':
     case '~':
     case '@':
     case ')':
-    case '[':
+
     case '{':
     case ';':
     case '}': return c;
 
+    case '[':
+      if( GOBBLE('?' ) )
+        return TOK_SAFE_START_INDEX;
+      return c;
+
     case '`':
     {
       char *tmp;
