@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 11bc949fa8a91d45a048d72f8067a1250a69335a $
 */
 
 /*
@@ -51,7 +50,6 @@ struct pike_string;
 
 unsigned long mpz_small_factor(mpz_t n, int limit);
 
-void mpz_next_prime(mpz_t p, mpz_t n, int count, int prime_limit);
 void my_mpz_xor _PROTO ((mpz_ptr, mpz_srcptr, mpz_srcptr));
 void get_mpz_from_digits(MP_INT *tmp,
 			 struct pike_string *digits,
@@ -108,13 +106,11 @@ static inline mp_limb_t MPZ_GETLIMBN (mpz_srcptr gmp_z, mp_size_t gmp_n)
 extern struct program *mpzmod_program;
 extern struct program *mpq_program;
 extern struct program *mpf_program;
-#ifdef AUTO_BIGNUM
 extern struct program *bignum_program;
-#endif
 
 #ifdef DEBUG_MALLOC
 #define get_mpz(S, THROW_ERROR, ARG_FUNC, ARG, ARGS)			\
-  ((S)->type <= MAX_REF_TYPE ? debug_malloc_touch((S)->u.object) : 0,	\
+  (REFCOUNTED_TYPE(TYPEOF(*(S))) ? debug_malloc_touch((S)->u.object) : 0, \
    debug_get_mpz((S), (THROW_ERROR), (ARG_FUNC), (ARG), (ARGS)))
 #else
 #define get_mpz debug_get_mpz 
@@ -126,12 +122,7 @@ extern struct program *bignum_program;
 #define OBTOMPQ(o) ((MP_RAT *)(o->storage))
 #define OBTOMPF(o) ((MP_FLT *)(o->storage))
 
-#ifdef AUTO_BIGNUM
-#define IS_MPZ_OBJ(O) ((O)->prog == bignum_program || (O)->prog == mpzmod_program)
-#else
-#define IS_MPZ_OBJ(O) ((O)->prog == mpzmod_program)
-#endif
-
+#define IS_MPZ_OBJ(O) ((O)->prog == bignum_program || (O)->prog == mpzmod_program || ((O)->storage && get_storage((O),mpzmod_program)==(O)->storage))
 #ifndef GMP_NUMB_BITS
 #define GMP_NUMB_BITS (SIZEOF_MP_LIMB_T * CHAR_BIT)
 #endif
