@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 9bec0a278688f3c4aaaddcf5af20b18a0de0922c $
 */
 
 #include "global.h"
@@ -12,16 +11,21 @@
 #include "interpret.h"
 #include "svalue.h"
 #include "threads.h"
+#include "array.h"
 #include "mapping.h"
 #include "pike_error.h"
 #include "stralloc.h"
 #include "builtin_functions.h"
 #include "operators.h"
 #include "module_support.h"
+#include "pike_types.h"
 
 #include "image.h"
 #include "colortable.h"
 
+#include "bignum.h"
+#include "bitvector.h"
+
 #ifdef __MINGW32__
 /* encodings.a will never contain a crc32 symbol. */
 #define DYNAMIC_MODULE
@@ -50,7 +54,6 @@ static struct pike_string *param_palette;
 static struct pike_string *param_spalette;
 static struct pike_string *param_image;
 static struct pike_string *param_alpha;
-static struct pike_string *param_type;
 static struct pike_string *param_bpp;
 static struct pike_string *param_background;
 static struct pike_string *param_zlevel;
@@ -167,8 +170,8 @@ static void image_png__chunk(INT32 args)
    struct pike_string *a,*b;
 
    if (args!=2 ||
-       sp[-args].type!=T_STRING ||
-       sp[1-args].type!=T_STRING)
+       TYPEOF(sp[-args]) != T_STRING ||
+       TYPEOF(sp[1-args]) != T_STRING)
       PIKE_ERROR("Image.PNG._chunk", "Illegal argument(s).\n", sp, args);
 
    a=sp[-args].u.string;
@@ -210,18 +213,18 @@ static void image_png___decode(INT32 args)
 {
    int nocrc=0;
    unsigned char *data;
+   struct array *res;
    size_t len;
    struct pike_string *str;
-   int n=0;
    ONERROR uwp;
 
    if (args<1)
-     SIMPLE_TOO_FEW_ARGS_ERROR("Image.PNG.__decode", 1);
-   if (sp[-args].type!=T_STRING)
-     SIMPLE_BAD_ARG_ERROR("Image.PNG.__decode", 1, "string");
+     SIMPLE_TOO_FEW_ARGS_ERROR("__decode", 1);
+   if (TYPEOF(sp[-args]) != T_STRING)
+     SIMPLE_BAD_ARG_ERROR("__decode", 1, "string");
 
    if (args>1 &&
-       (sp[1-args].type!=T_INT ||
+       (TYPEOF(sp[1-args]) != T_INT ||
 	sp[1-args].u.integer!=0))
       nocrc=1;
 
@@ -249,6 +252,9 @@ static void image_png___decode(INT32 args)
    SET_ONERROR(uwp,do_free_string,str);
    len-=8; data+=8;
 
+   check_stack(20);
+   BEGIN_AGGREGATE_ARRAY(10);
+
    while (len>8)
    {
       unsigned long x;
@@ -256,30 +262,36 @@ static void image_png___decode(INT32 args)
       push_string(make_shared_binary_string((char*)data+4,4));
       len-=8;
       data+=8;
+
       if (x>len)
       {
-	 push_string(make_shared_binary_string((char*)data,len));
-	 push_int(0);
-	 f_aggregate(3);
-	 n++;
-	 break;
+	push_string(make_shared_binary_string((char*)data,len));
+	push_int(0);
+	f_aggregate(3);
+	DO_AGGREGATE_ARRAY(20);
+        break;
       }
+
       push_string(make_shared_binary_string((char*)data,x));
       if (!nocrc && x+4<=len)
-	 push_int( crc32(crc32(0,NULL,0),data-4,x+4) ==
-		   int_from_32bit(data+x) );
+	push_int( crc32(crc32(0,NULL,0),data-4,x+4) ==
+                  int_from_32bit(data+x) );
       else
-	 push_int(0);
+	push_int(0);
       f_aggregate(3);
-      n++;
+      DO_AGGREGATE_ARRAY(20);
+
       if (x+4>len) break;
+
+      if( int_from_32bit((unsigned char *)data) == 0x49454e44 ) /* IEND */
+        break;
+
       len-=x+4;
       data+=x+4;
    }
+   CALL_AND_UNSET_ONERROR(uwp);
 
-   UNSET_ONERROR(uwp);
-   free_string(str);
-   f_aggregate(n);
+   END_AGGREGATE_ARRAY;
 }
 
 /*! @decl mapping _decode(string|array data)
@@ -425,6 +437,15 @@ static struct pike_string *_png_unfilter(unsigned char *data,
       if (!len || !ysize--)
       {
 	 if (pos) *pos=s;
+         if (d - STR0(ps) < ps->len) {
+            /* Should we throw an error here, instead?
+             * This case means that there is extra
+             * IDAT data, when we have already processed all
+             * scanlines specified in the IHDR
+             *  /arne
+             */
+            memset(d, 0, ps->len - (d - STR0(ps)));
+         }
 	 return end_shared_string(ps);
       }
       x=xsize;
@@ -518,6 +539,7 @@ static struct pike_string *_png_unfilter(unsigned char *data,
 
 	    break;
 	 default:
+            free_string(ps);
 	    Pike_error("Unsupported subfilter %d (filter %d)\n", s[-1],type);
       }
    }
@@ -807,7 +829,7 @@ static int _png_write_rgb(rgb_group *w1,
 			i-=4;
 			if (x)
 			{
-			   int m=((*s)>>i)&3;
+			   int m=((*s)>>i)&15;
 			   x--;
 			   *(d1++)=ct->u.flat.entries[CUTPLTE(m,mz)].color;
 			   if (m>=trns->len)
@@ -991,12 +1013,19 @@ static int _png_decode_idat(struct IHDR *ihdr, struct neo_colortable *ct,
   ONERROR err, a_err;
 
   png_decompress(ihdr->compression);
-  if( sp[-1].type!=T_STRING )
+  if( TYPEOF(sp[-1]) != T_STRING )
     Pike_error("Got illegal data from decompression.\n");
 
-  w1=xalloc(sizeof(rgb_group)*ihdr->width*ihdr->height);
+  if (INT32_MUL_OVERFLOW(ihdr->width, ihdr->height) ||
+      ihdr->width*ihdr->height > (0x7fffffff - RGB_VEC_PAD)/sizeof(rgb_group)) {
+    /* Overflow in size calculation below. */
+    Pike_error("Image too large (%d * %d)\n",
+	       ihdr->width, ihdr->height);
+  }
+
+  w1=xalloc(sizeof(rgb_group)*ihdr->width*ihdr->height + RGB_VEC_PAD);
   SET_ONERROR(err, free_and_clear, &w1);
-  wa1=xalloc(sizeof(rgb_group)*ihdr->width*ihdr->height);
+  wa1=xalloc(sizeof(rgb_group)*ihdr->width*ihdr->height + RGB_VEC_PAD);
   SET_ONERROR(a_err, free_and_clear, &wa1);
 
   fs = sp[-1].u.string;
@@ -1011,6 +1040,8 @@ static int _png_decode_idat(struct IHDR *ihdr, struct neo_colortable *ct,
                      ihdr->filter,ihdr->type,ihdr->bpp,
                      NULL);
 
+    push_string(fs);
+
     if (!_png_write_rgb(w1,wa1,
                         ihdr->type,ihdr->bpp,
                         (unsigned char*)fs->str,fs->len,
@@ -1021,7 +1052,7 @@ static int _png_decode_idat(struct IHDR *ihdr, struct neo_colortable *ct,
       free(wa1);
       wa1=NULL;
     }
-    free_string(fs);
+    pop_stack();
     break;
 
   case 1: /* adam7 */
@@ -1031,9 +1062,9 @@ static int _png_decode_idat(struct IHDR *ihdr, struct neo_colortable *ct,
       int got_alpha = 0;
 
       /* need arena */
-      t1=xalloc(sizeof(rgb_group)*ihdr->width*ihdr->height);
+      t1=xalloc(sizeof(rgb_group)*ihdr->width*ihdr->height + RGB_VEC_PAD);
       SET_ONERROR(t_err, free_and_clear, &t1);
-      ta1=xalloc(sizeof(rgb_group)*ihdr->width*ihdr->height);
+      ta1=xalloc(sizeof(rgb_group)*ihdr->width*ihdr->height + RGB_VEC_PAD);
       SET_ONERROR(ta_err, free_and_clear, &ta1);
 
       /* loop over adam7 interlace's
@@ -1058,6 +1089,8 @@ static int _png_decode_idat(struct IHDR *ihdr, struct neo_colortable *ct,
 			 ihdr->filter,ihdr->type,ihdr->bpp,
 			 &s0);
 
+        push_string(ds);
+
 	if (_png_write_rgb(w1,wa1,ihdr->type,ihdr->bpp,
 			   (unsigned char*)ds->str,ds->len,
 			   iwidth,
@@ -1075,7 +1108,7 @@ static int _png_decode_idat(struct IHDR *ihdr, struct neo_colortable *ct,
 	  for (x=x0; x<ihdr->width; x+=xd)
 	    t1[x+y*ihdr->width]=*(d1++);
 
-	free_string(ds);
+        pop_stack();
       }
 
       free(wa1);
@@ -1103,7 +1136,7 @@ static int _png_decode_idat(struct IHDR *ihdr, struct neo_colortable *ct,
 
   /* Create image object and leave it on the stack */
   push_object(clone_object(image_program,0));
-  img=(struct image*)get_storage(sp[-1].u.object,image_program);
+  img=get_storage(sp[-1].u.object,image_program);
   if (img->img) free(img->img); /* protect from memleak */
   img->xsize=ihdr->width;
   img->ysize=ihdr->height;
@@ -1113,7 +1146,7 @@ static int _png_decode_idat(struct IHDR *ihdr, struct neo_colortable *ct,
   if( wa1 )
   {
     push_object(clone_object(image_program,0));
-    img=(struct image*)get_storage(sp[-1].u.object,image_program);
+    img=get_storage(sp[-1].u.object,image_program);
     if (img->img) free(img->img); /* protect from memleak */
     img->xsize=ihdr->width;
     img->ysize=ihdr->height;
@@ -1144,39 +1177,32 @@ static void img_png_decode(INT32 args, int mode)
    ONERROR err;
 
    if (args<1)
-     SIMPLE_TOO_FEW_ARGS_ERROR("Image.PNG._decode", 1);
+     SIMPLE_TOO_FEW_ARGS_ERROR("_decode", 1);
 
    m=allocate_mapping(10);
    push_mapping(m);
 
    if (args>=2)
    {
-      if (sp[1-args-1].type!=T_MAPPING)
-	SIMPLE_BAD_ARG_ERROR("Image.PNG._decode", 2, "mapping");
+      if (TYPEOF(sp[1-args-1]) != T_MAPPING)
+	SIMPLE_BAD_ARG_ERROR("_decode", 2, "mapping");
 
       push_svalue(sp+1-args-1);
       ref_push_string(param_palette);
       f_index(2);
-      switch (sp[-1].type)
+      switch (TYPEOF(sp[-1]))
       {
-	 case T_OBJECT:
-	    push_text("cast");
-	    if (sp[-1].type==T_INT)
-	       PIKE_ERROR("Image.PNG._decode",
-			  "Illegal value of option \"palette\".\n",
-			  sp, args);
-	    f_index(2);
-	    push_text("array");
-	    f_call_function(2);
 	 case T_ARRAY:
 	 case T_STRING:
 	    push_object(clone_object(image_colortable_program,1));
 
-	    ct=(struct neo_colortable*)get_storage(sp[-1].u.object,
-						   image_colortable_program);
+            /* Fallthrough */
+	 case T_OBJECT:
+	    ct=get_storage(sp[-1].u.object,
+                           image_colortable_program);
 	    if (!ct)
-	       PIKE_ERROR("Image.PNG._decode",
-			  "Internal error: cloned colortable isn't colortable.\n", sp, args);
+	       PIKE_ERROR("_decode",
+			  "Object isn't colortable.\n", sp, args);
 	    mapping_string_insert(m, param_palette, sp-1);
 	    pop_stack();
 	    break;
@@ -1184,7 +1210,7 @@ static void img_png_decode(INT32 args, int mode)
 	    pop_stack();
 	    break;
 	 default:
-	    PIKE_ERROR("Image.PNG._decode",
+	    PIKE_ERROR("_decode",
 		       "Illegal value of option \"palette\".\n",
 		       sp, args);
       }
@@ -1195,15 +1221,15 @@ static void img_png_decode(INT32 args, int mode)
    push_mapping(m);
    stack_swap();
 
-   if (sp[-1].type==T_STRING)
+   if (TYPEOF(sp[-1]) == T_STRING)
    {
       push_int(1); /* no care crc */
       image_png___decode(2);
-      if (sp[-1].type!=T_ARRAY)
-	 PIKE_ERROR("Image.PNG._decode", "Not PNG data.\n", sp ,args);
+      if (TYPEOF(sp[-1]) != T_ARRAY)
+	 PIKE_ERROR("_decode", "Not PNG data.\n", sp ,args);
    }
-   else if (sp[-1].type!=T_ARRAY)
-     SIMPLE_BAD_ARG_ERROR("Image.PNG._decode", 1, "string");
+   else if (TYPEOF(sp[-1]) != T_ARRAY)
+     SIMPLE_BAD_ARG_ERROR("_decode", 1, "string");
 
    a=sp[-1].u.array;
 
@@ -1214,10 +1240,10 @@ static void img_png_decode(INT32 args, int mode)
       size_t len;
 
 #ifdef PIKE_DEBUG
-      if (a->item[i].type!=T_ARRAY ||
+      if (TYPEOF(a->item[i]) != T_ARRAY ||
 	  (b=a->item[i].u.array)->size!=3 ||
-	  b->item[0].type!=T_STRING ||
-	  b->item[1].type!=T_STRING ||
+	  TYPEOF(b->item[0]) != T_STRING ||
+	  TYPEOF(b->item[1]) != T_STRING ||
 	  b->item[0].u.string->len!=4)
 	 Pike_error("Image.PNG._decode: Illegal stuff in array index %d\n",i);
 #else
@@ -1230,27 +1256,31 @@ static void img_png_decode(INT32 args, int mode)
       if (!i &&
 	  int_from_32bit((unsigned char*)b->item[0].u.string->str)
 	  != 0x49484452 )
-	 PIKE_ERROR("Imge.PNG.decode", "First chunk isn't IHDR.\n",
+	 PIKE_ERROR("_decode", "First chunk isn't IHDR.\n",
 		    sp, args);
 
       switch (int_from_32bit((unsigned char*)b->item[0].u.string->str))
       {
 	 /* ------ major chunks ------------ */
-         case 0x49484452: /* IHDR */
+         case 0x49484452: { /* IHDR */
+            size_t bytes;
 	    /* header info */
-	    if (b->item[1].u.string->len!=13)
-	       PIKE_ERROR("Image.PNG._decode",
+	    if (len!=13)
+	       PIKE_ERROR("_decode",
 			  "Illegal header (IHDR chunk).\n", sp, args);
 
 	    ihdr.width=int_from_32bit(data+0);
 	    ihdr.height=int_from_32bit(data+4);
 
-            if( sizeof(rgb_group)*(double)ihdr.width*(double)ihdr.height >
-                (double)INT_MAX )
+            if (!ihdr.width || !ihdr.height)
+              Pike_error("Image.PNG._decode: Invalid dimensions in IHDR chunk.\n");
+
+            if (DO_SIZE_T_MUL_OVERFLOW(ihdr.width, ihdr.height, &bytes) ||
+                DO_SIZE_T_MUL_OVERFLOW(bytes, sizeof(rgb_group), &bytes) ||
+                bytes > INT_MAX) {
               Pike_error("Image.PNG._decode: Too large image "
-                         "(total size exceeds %d bytes (%.0f))\n", INT_MAX,
-                         sizeof(rgb_group)*
-                         (double)ihdr.width*(double)ihdr.height);
+                         "(total size exceeds %d bytes)\n", INT_MAX);
+            }
 
 	    ihdr.bpp=data[8];
 	    ihdr.type=data[9];
@@ -1258,7 +1288,7 @@ static void img_png_decode(INT32 args, int mode)
 	    ihdr.filter=data[11];
 	    ihdr.interlace=data[12];
 	    break;
-
+         }
          case 0x504c5445: /* PLTE */
 	    /* palette info, 3n bytes */
 
@@ -1269,8 +1299,7 @@ static void img_png_decode(INT32 args, int mode)
 
 	    if (ihdr.type==3)
 	    {
-	       ct=(struct neo_colortable*)
-		  get_storage(sp[-1].u.object,image_colortable_program);
+	       ct=get_storage(sp[-1].u.object,image_colortable_program);
 	       mapping_string_insert(m, param_palette, sp-1);
 	    }
 	    else
@@ -1299,10 +1328,9 @@ static void img_png_decode(INT32 args, int mode)
 	  {
 	    int i;
             if(mode==MODE_IMAGE_ONLY) break;
-	    if(b->item[1].u.string->len!=32) break;
+	    if(len!=32) break;
 	    for(i=0; i<32; i+=4)
-	      push_float((float)int_from_32bit((unsigned char*)b->
-                                        item[1].u.string->str+i)/100000.0);
+	      push_float((float)int_from_32bit(data+i)/100000.0);
 	    f_aggregate(8);
 	    push_text("chroma");
 	    mapping_insert(m,sp-1,sp-2);
@@ -1312,12 +1340,14 @@ static void img_png_decode(INT32 args, int mode)
 
           case 0x73424954: /* sBIT */
 	  {
-	    int i;
+	    size_t i;
             if(mode==MODE_IMAGE_ONLY) break;
-	    for(i=0; i<b->item[1].u.string->len; i++)
-	      push_int(b->item[1].u.string->str[i]);
-	    f_aggregate(b->item[1].u.string->len);
-	    push_constant_text("sbit");
+            /* sBIT chunks are not longer than 4 bytes */
+            if (len > 4) break;
+	    for(i=0; i<len; i++)
+	      push_int(data[i]);
+	    f_aggregate(len);
+	    push_text("sbit");
 	    mapping_insert(m,sp-1,sp-2);
 	    pop_n_elems(2);
 	  }
@@ -1325,53 +1355,63 @@ static void img_png_decode(INT32 args, int mode)
 
           case 0x67414d41: /* gAMA */
             if(mode==MODE_IMAGE_ONLY) break;
-	    if(b->item[1].u.string->len!=4) break;
-	    push_constant_text("gamma");
-	    push_float((float)int_from_32bit((unsigned char*)b->
-                                      item[1].u.string->str)/100000.0);
+	    if(len!=4) break;
+	    push_text("gamma");
+	    push_float((float)int_from_32bit(data)/100000.0);
 	    mapping_insert(m,sp-2,sp-1);
 	    pop_n_elems(2);
 	    break;
 
-          case 0x70485973: /* pHYs */
+          case 0x70485973: { /* pHYs */
+            int tmp1, tmp2;
             if(mode==MODE_IMAGE_ONLY) break;
-	    if(b->item[1].u.string->len!=9) break;
-	    push_int(b->item[1].u.string->str[8]);
-	    push_int(int_from_32bit((unsigned char*)b->
-                                    item[1].u.string->str));
-	    push_int(int_from_32bit((unsigned char*)b->
-                                    item[1].u.string->str+4));
+	    if(len!=9) break;
+            tmp1 = get_unaligned_be32(data);
+            tmp2 = get_unaligned_be32(data+4);
+            /* the image dimensions are valid in the range
+             * 0 .. MAX_INT32
+             */
+            if (data[8] != 0 && data[8] != 1) break;
+            if (tmp1 < 0 || tmp2 < 0) break;
+	    push_int(data[8]);
+	    push_int(tmp1);
+	    push_int(tmp2);
 	    f_aggregate(3);
-	    push_constant_text("physical");
+	    push_text("physical");
 	    mapping_insert(m,sp-1,sp-2);
 	    pop_n_elems(2);
 	    break;
-
-          case 0x6f464673: /* oFFs */
+          }
+          case 0x6f464673: { /* oFFs */
+            int tmp1, tmp2;
             if(mode==MODE_IMAGE_ONLY) break;
-	    if(b->item[1].u.string->len!=9) break;
-	    push_int(b->item[1].u.string->str[8]);
-	    push_int(int_from_32bit((unsigned char*)b->
-                                    item[1].u.string->str));
-	    push_int(int_from_32bit((unsigned char*)b->
-                                    item[1].u.string->str+4));
+	    if(len!=9) break;
+            tmp1 = get_unaligned_be32(data);
+            tmp2 = get_unaligned_be32(data+4);
+            /* the oFFs image offsets are valid in the range
+             * of -MAX_INT32 .. MAX_INT32 */
+            if (data[8] != 0 && data[8] != 1) break;
+            if (tmp1 == MIN_INT32 || tmp2 == MIN_INT32) break;
+	    push_int(data[8]);
+	    push_int(tmp1);
+	    push_int(tmp2);
 	    f_aggregate(3);
-	    push_constant_text("offset");
+	    push_text("offset");
 	    mapping_insert(m,sp-1,sp-2);
 	    pop_n_elems(2);
 	    break;
-
+          }
           case 0x74494d45: /* tIME */
             if(mode==MODE_IMAGE_ONLY) break;
-	    if(b->item[1].u.string->len!=7) break;
-	    push_int(int_from_16bit(b->item[1].u.string->str));
-	    push_int(b->item[1].u.string->str[2]);
-	    push_int(b->item[1].u.string->str[3]);
-	    push_int(b->item[1].u.string->str[4]);
-	    push_int(b->item[1].u.string->str[5]);
-	    push_int(b->item[1].u.string->str[6]);
+	    if(len!=7) break;
+	    push_int(int_from_16bit(data));
+	    push_int(data[2]);
+	    push_int(data[3]);
+	    push_int(data[4]);
+	    push_int(data[5]);
+	    push_int(data[6]);
 	    f_aggregate(6);
-	    push_constant_text("time");
+	    push_text("time");
 	    mapping_insert(m,sp-1,sp-2);
 	    pop_n_elems(2);
 	    break;
@@ -1382,9 +1422,9 @@ static void img_png_decode(INT32 args, int mode)
 	    {
 	       case 0:
 	       case 4:
-		  if (b->item[1].u.string->len==2)
+		  if (len==2)
 		  {
-		     int z=_png_c16(b->item[1].u.string->str[0],ihdr.bpp);
+		     int z=_png_c16(data[0],ihdr.bpp);
 		     push_int(z);
 		     push_int(z);
 		     push_int(z);
@@ -1393,25 +1433,24 @@ static void img_png_decode(INT32 args, int mode)
 		     continue;
 		  break;
 	       case 3:
-		  if (b->item[1].u.string->len!=1 ||
+		  if (len!=1 ||
 		      !ct || ct->type!=NCT_FLAT ||
-		      b->item[1].u.string->str[0] >=
-		      ct->u.flat.numentries)
+		      data[0] >= ct->u.flat.numentries)
 		     continue;
 		  else
 		  {
-		     push_int(ct->u.flat.entries[(int)b->item[1].u.string->str[0]].color.r);
-		     push_int(ct->u.flat.entries[(int)b->item[1].u.string->str[0]].color.g);
-		     push_int(ct->u.flat.entries[(int)b->item[1].u.string->str[0]].color.b);
+		     push_int(ct->u.flat.entries[data[0]].color.r);
+		     push_int(ct->u.flat.entries[data[0]].color.g);
+		     push_int(ct->u.flat.entries[data[0]].color.b);
 		  }
                   break;
 	       default:
-		  if (b->item[1].u.string->len==6)
+		  if (len==6)
 		  {
 		     int j;
 		     for (j=0; j<3; j++)
 		     {
-			int z=_png_c16(b->item[1].u.string->str[j*2],ihdr.bpp);
+			int z=_png_c16(data[j*2],ihdr.bpp);
 			push_int(z);
 		     }
 		  } else
@@ -1424,8 +1463,10 @@ static void img_png_decode(INT32 args, int mode)
 	    break;
 
          case 0x74524e53: /* tRNS */
+            if (trns) break;
 	    trns=b->item[1].u.string;
             add_ref(trns);
+            SET_ONERROR(err, png_free_string, trns);
 	    break;
 
          default:
@@ -1436,19 +1477,16 @@ static void img_png_decode(INT32 args, int mode)
 
 	    ref_push_string(b->item[1].u.string);
 	    ref_push_string(b->item[0].u.string);
-	    mapping_insert(m,sp-1,sp-2);
+            /* do not replace existing entries */
+	    low_mapping_insert(m,sp-1,sp-2,0);
 	    pop_n_elems(2);
       }
    }
 
-
    /* on stack: mapping   nstring */
 
    if ( mode != MODE_HEADER_ONLY )
    {
-     if(trns)
-       SET_ONERROR(err, png_free_string, &trns);
-
      if (ihdr.type==-1)
        PIKE_ERROR("Image.PNG._decode", "Missing header (IHDR chunk).\n",
                   sp, args);
@@ -1471,29 +1509,29 @@ static void img_png_decode(INT32 args, int mode)
      mapping_string_insert(m, param_image, sp-1);
      pop_stack();
 
-     if(trns)
-       UNSET_ONERROR(err);
    }
 
-   if (trns)
+   if (trns) {
+     UNSET_ONERROR(err);
      png_free_string (trns);
+   }
 
    if ( mode != MODE_IMAGE_ONLY )
    {
      push_int(ihdr.type);
-     mapping_string_insert(m, param_type, sp-1);
+     mapping_string_insert(m, literal_type_string, sp-1);
      pop_stack();
 
      push_int(ihdr.bpp);
      mapping_string_insert(m, param_bpp, sp-1);
      pop_stack();
 
-     push_constant_text("xsize");
+     push_text("xsize");
      push_int(ihdr.width);
      mapping_insert(m,sp-2,sp-1);
      pop_n_elems(2);
 
-     push_constant_text("ysize");
+     push_text("ysize");
      push_int(ihdr.height);
      mapping_insert(m,sp-2,sp-1);
      pop_n_elems(2);
@@ -1543,45 +1581,43 @@ static void image_png_encode(INT32 args)
    char buf[20];
 
    if (!args)
-     SIMPLE_TOO_FEW_ARGS_ERROR("Image.PNG.encode", 1);
+     SIMPLE_TOO_FEW_ARGS_ERROR("encode", 1);
 
-   if (sp[-args].type!=T_OBJECT ||
-       !(img=(struct image*)
-	 get_storage(sp[-args].u.object,image_program)))
-     SIMPLE_BAD_ARG_ERROR("Image.PNG.encode", 1, "Image.Image");
+   if (TYPEOF(sp[-args]) != T_OBJECT ||
+       !(img=get_storage(sp[-args].u.object,image_program)))
+     SIMPLE_BAD_ARG_ERROR("encode", 1, "Image.Image");
 
    if (!img->img)
-      PIKE_ERROR("Image.PNG.encode", "No image.\n", sp, args);
+      PIKE_ERROR("encode", "No image.\n", sp, args);
 
-   if (args>1)
+   if ((args>1) && !IS_UNDEFINED(sp + 1-args))
    {
      struct svalue *s;
 
-     if (sp[1-args].type!=T_MAPPING)
-	SIMPLE_BAD_ARG_ERROR("Image.PNG.encode", 2, "mapping");
+     if (TYPEOF(sp[1-args]) != T_MAPPING)
+	SIMPLE_BAD_ARG_ERROR("encode", 2, "mapping");
 
       /* Attribute alpha */
       s = low_mapping_string_lookup(sp[1-args].u.mapping, param_alpha);
 
       if( s )
       {
-        if( s->type==T_OBJECT &&
-            (alpha=(struct image*)
-             get_storage(s->u.object,image_program)) )
+        if( TYPEOF(*s) == T_OBJECT &&
+            (alpha=get_storage(s->u.object,image_program)) )
         {
           if (alpha->xsize!=img->xsize ||
               alpha->ysize!=img->ysize)
-            PIKE_ERROR("Image.PNG.encode",
+            PIKE_ERROR("encode",
                        "Option (arg 2) \"alpha\"; images differ in size.\n",
                        sp, args);
 
           if (!alpha->img)
-            PIKE_ERROR("Image.PNG.encode",
+            PIKE_ERROR("encode",
                        "Option (arg 2) \"alpha\"; no image\n",
                        sp, args);
         }
-        else if( !(s->type==T_INT && s->u.integer==0) )
-          PIKE_ERROR("Image.PNG.encode",
+        else if( !(TYPEOF(*s) == T_INT && s->u.integer==0) )
+          PIKE_ERROR("encode",
                      "Option (arg 2) \"alpha\" has illegal type.\n",
                      sp, args);
       }
@@ -1589,11 +1625,10 @@ static void image_png_encode(INT32 args)
       /* Attribute palette */
       s = low_mapping_string_lookup(sp[1-args].u.mapping, param_palette);
 
-      if (s && !(s->type==T_INT && s->u.integer==0))
-	 if (s->type!=T_OBJECT ||
-	     !(ct=(struct neo_colortable*)
-	       get_storage(s->u.object,image_colortable_program)))
-	   PIKE_ERROR("Image.PNG.encode",
+      if (s && !(TYPEOF(*s) == T_INT && s->u.integer==0))
+	 if (TYPEOF(*s) != T_OBJECT ||
+	     !(ct=get_storage(s->u.object,image_colortable_program)))
+	   PIKE_ERROR("encode",
 		      "Option (arg 2) \"palette\" has illegal type.\n",
 		      sp, args);
 
@@ -1602,8 +1637,8 @@ static void image_png_encode(INT32 args)
 
       if ( s )
       {
-        if( s->type!=T_INT )
-          PIKE_ERROR("Image.PNG.encode",
+        if( TYPEOF(*s) != T_INT )
+          PIKE_ERROR("encode",
                      "Option (arg 2) \"zlevel\" has illegal value.\n",
                      sp, args);
         else
@@ -1614,8 +1649,8 @@ static void image_png_encode(INT32 args)
       s = low_mapping_string_lookup(sp[1-args].u.mapping, param_zstrategy);
       if( s )
       {
-        if ( s->type!=T_INT )
-          PIKE_ERROR("Image.PNG.encode",
+        if ( TYPEOF(*s) != T_INT )
+          PIKE_ERROR("encode",
                      "Option (arg 2) \"zstrategy\" has illegal value.\n",
                      sp, args);
         else
@@ -1633,7 +1668,7 @@ static void image_png_encode(INT32 args)
       ptrdiff_t sz;
       sz = image_colortable_size(ct);
       if (sz>256)
-	 PIKE_ERROR("Image.PNG.encode", "Palette size to large; "
+	 PIKE_ERROR("encode", "Palette size to large; "
 		    "PNG doesn't support bigger palettes then 256 colors.\n",
 		    sp, args);
       if (sz>16) bpp=8;
@@ -1663,7 +1698,7 @@ static void image_png_encode(INT32 args)
    {
       struct pike_string *ps;
       ps=begin_shared_string(3<<bpp);
-      MEMSET(ps->str,0,3<<bpp);
+      memset(ps->str,0,3<<bpp);
       image_colortable_write_rgb(ct,(unsigned char*)ps->str);
       push_string(end_shared_string(ps));
       push_png_chunk("PLTE",NULL);
@@ -1675,7 +1710,7 @@ static void image_png_encode(INT32 args)
    if (alpha) sa=alpha->img;
    if (ct) {
       if (alpha) {
-	 PIKE_ERROR("Image.PNG.encode",
+	 PIKE_ERROR("encode",
 		    "Colortable and alpha channel not supported "
 		    "at the same time.\n", sp, args);
       }
@@ -1686,7 +1721,7 @@ static void image_png_encode(INT32 args)
 	 unsigned char *tmp, *ts;
 
          x = img->xsize;
-         tmp=xalloc(x*y);
+	 tmp=xalloc(x*y + RGB_VEC_PAD);
 
 	 image_colortable_index_8bit_image(ct,s,tmp,x*y,x);
          ps=begin_shared_string( y * ((x*bpp+7)/8+1) );
@@ -1699,7 +1734,7 @@ static void image_png_encode(INT32 args)
 	    *(d++)=0; /* filter */
 	    if (bpp==8)
 	    {
-	       MEMCPY(d,ts,x);
+	       memcpy(d,ts,x);
 	       ts += x;
                d += x;
 	    }
@@ -1810,10 +1845,10 @@ static void image_png_decode_header(INT32 args)
 static void image_png_decode(INT32 args)
 {
    if (!args)
-     SIMPLE_TOO_FEW_ARGS_ERROR("Image.PNG.decode", 1);
+     SIMPLE_TOO_FEW_ARGS_ERROR("decode", 1);
 
    img_png_decode(args, MODE_IMAGE_ONLY);
-   push_constant_text("image");
+   push_text("image");
    f_index(2);
 }
 
@@ -1830,20 +1865,20 @@ static void image_png_decode_alpha(INT32 args)
 {
    struct svalue s;
    if (!args)
-     SIMPLE_TOO_FEW_ARGS_ERROR("Image.PNG.decode_alpha", 1);
+     SIMPLE_TOO_FEW_ARGS_ERROR("decode_alpha", 1);
 
    image_png__decode(args);
    assign_svalue_no_free(&s,sp-1);
-   push_constant_text("alpha");
+   push_text("alpha");
    f_index(2);
 
-   if (sp[-1].type==T_INT)
+   if (TYPEOF(sp[-1]) == T_INT)
    {
       push_svalue(&s);
-      push_constant_text("xsize");
+      push_text("xsize");
       f_index(2);
       push_svalue(&s);
-      push_constant_text("ysize");
+      push_text("ysize");
       f_index(2);
       push_int(255);
       push_int(255);
@@ -1869,7 +1904,6 @@ void exit_image_png(void)
    free_string(param_alpha);
    free_string(param_bpp);
    free_string(param_background);
-   free_string(param_type);
    free_string(param_zlevel);
    free_string(param_zstrategy);
 }
@@ -1886,7 +1920,7 @@ void init_image_png(void)
 #else
    push_text("Gz.inflate");
    SAFE_APPLY_MASTER("resolv",1);
-   if( Pike_sp[-1].type==T_PROGRAM )
+   if( TYPEOF(Pike_sp[-1]) == T_PROGRAM )
      gz = 1;
    pop_stack();
 #endif
@@ -1920,7 +1954,6 @@ void init_image_png(void)
    param_image=make_shared_string("image");
    param_alpha=make_shared_string("alpha");
    param_bpp=make_shared_string("bpp");
-   param_type=make_shared_string("type");
    param_background=make_shared_string("background");
    param_zlevel=make_shared_string("zlevel");
    param_zstrategy=make_shared_string("zstrategy");
