@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: a7af4dee2d3e783b98a7df659562c9bb3d598ba8 $
 */
 
 /*
@@ -10,9 +9,8 @@
 **! submodule X
 **!
 **!	This submodule handles encoding and decoding of
-**!	the binary formats of X11.
-**!
-**!	
+**! the binary formats of X11, and can also be used for other
+**! non-compressed image formats.
 **!
 **! see also: Image, Image.Image, Image.Colortable
 */
@@ -145,65 +143,64 @@ static void image_x_encode_truecolor(INT32 args)
    if (args<10)
       Pike_error("Image.X.encode_truecolor: too few arguments (expected 10 arguments)\n");
    
-   if (sp[-args].type!=T_OBJECT ||
-       !(img=(struct image*)get_storage(sp[-args].u.object,image_program)))
+   if (TYPEOF(sp[-args]) != T_OBJECT ||
+       !(img=get_storage(sp[-args].u.object,image_program)))
       Pike_error("Image.X.encode_truecolor: illegal argument 1 (expected image object)\n");
    if (args>10)
-      if (sp[10-args].type!=T_OBJECT ||
-	  !(nct=(struct neo_colortable*)
-	    get_storage(sp[10-args].u.object,image_colortable_program)))
+      if (TYPEOF(sp[10-args]) != T_OBJECT ||
+	  !(nct=get_storage(sp[10-args].u.object,image_colortable_program)))
 	 Pike_error("Image.X.encode_truecolor: illegal argument 10 (expected colortable object)\n");
 	 
-   if (sp[1-args].type!=T_INT)
+   if (TYPEOF(sp[1-args]) != T_INT)
       Pike_error("Image.X.encode_truecolor: illegal argument 2 (expected integer)\n");
    else
       bpp=sp[1-args].u.integer;
 
-   if (sp[2-args].type!=T_INT)
+   if (TYPEOF(sp[2-args]) != T_INT)
       Pike_error("Image.X.encode_truecolor: illegal argument 3 (expected integer)\n");
    else
       alignbits=sp[2-args].u.integer;
 
    if (!alignbits) alignbits=1;
 
-   if (sp[3-args].type!=T_INT)
+   if (TYPEOF(sp[3-args]) != T_INT)
       Pike_error("Image.X.encode_truecolor: illegal argument 4 (expected integer)\n");
    else
       swap_bytes=sp[3-args].u.integer;
 
-   if (sp[4-args].type!=T_INT)
+   if (TYPEOF(sp[4-args]) != T_INT)
       Pike_error("Image.X.encode_truecolor: illegal argument 5 (expected integer)\n");
    else
       rbits=sp[4-args].u.integer;
 
-   if (sp[5-args].type!=T_INT)
+   if (TYPEOF(sp[5-args]) != T_INT)
       Pike_error("Image.X.encode_truecolor: illegal argument 6 (expected integer)\n");
    else
       rshift=sp[5-args].u.integer;
 
-   if (sp[6-args].type!=T_INT)
+   if (TYPEOF(sp[6-args]) != T_INT)
       Pike_error("Image.X.encode_truecolor: illegal argument 7 (expected integer)\n");
    else
       gbits=sp[6-args].u.integer;
 
-   if (sp[7-args].type!=T_INT)
+   if (TYPEOF(sp[7-args]) != T_INT)
       Pike_error("Image.X.encode_truecolor: illegal argument 8 (expected integer)\n");
    else
       gshift=sp[7-args].u.integer;
 
-   if (sp[8-args].type!=T_INT)
+   if (TYPEOF(sp[8-args]) != T_INT)
       Pike_error("Image.X.encode_truecolor: illegal argument 9 (expected integer)\n");
    else
       bbits=sp[8-args].u.integer;
 
-   if (sp[9-args].type!=T_INT)
+   if (TYPEOF(sp[9-args]) != T_INT)
       Pike_error("Image.X.encode_truecolor: illegal argument 10 (expected integer)\n");
    else
       bshift=sp[9-args].u.integer;
 
    if (nct) 
    {
-      tmp=(rgb_group*)xalloc(sizeof(rgb_group)*img->xsize*img->ysize +1);
+      tmp=xalloc(sizeof(rgb_group)*img->xsize*img->ysize +RGB_VEC_PAD);
       if (!image_colortable_map_image(nct,img->img,tmp,
 				      img->xsize*img->ysize,img->xsize))
       {
@@ -260,7 +257,7 @@ static void image_x_encode_truecolor(INT32 args)
       }
       else
       {
-	 MEMSET(d,0,( ( (img->xsize*bpp+alignbits-1) / alignbits )
+	 memset(d,0,( ( (img->xsize*bpp+alignbits-1) / alignbits )
 		      * alignbits*img->ysize  +7 ) / 8);
 	 while (y--)
 	 {
@@ -369,12 +366,12 @@ static void image_x_encode_truecolor(INT32 args)
    push_string(end_shared_string(dest));
 }
 
-static INLINE void image_x_examine_mask(struct svalue *mask,
+static void image_x_examine_mask(struct svalue *mask,
 					const char *what,
 					int *bits,int *shift)
 {
    unsigned long x;
-   if (mask->type!=T_INT)
+   if (TYPEOF(*mask) != T_INT)
       Pike_error("Image.X.encode_truecolor_masks: illegal %s (expected integer)\n",what);
 
    x=(unsigned long)mask->u.integer;
@@ -394,7 +391,7 @@ static INLINE void image_x_examine_mask(struct svalue *mask,
 static void image_x_call_examine_mask(INT32 args)
 {
    int bits,shift;
-   if (args<1 || sp[-args].type!=T_INT)
+   if (args<1 || TYPEOF(sp[-args]) != T_INT)
       Pike_error("Image.X.examine_mask: illegal argument(s)\n");
 
    image_x_examine_mask(sp-args,"argument 1",&bits,&shift);
@@ -412,21 +409,21 @@ static void image_x_encode_truecolor_masks(INT32 args)
 
    if (args<7) 
       Pike_error("Image.X.encode_truecolor_masks: too few arguments (expected 7 arguments)\n");
-   if (sp[-args].type!=T_OBJECT ||
+   if (TYPEOF(sp[-args]) != T_OBJECT ||
        !get_storage(sp[-args].u.object,image_program))
       Pike_error("Image.X.encode_truecolor_masks: illegal argument 1 (expected image object)\n");
 
    if (args>7)
-      if (sp[7-args].type!=T_OBJECT ||
+       if (TYPEOF(sp[7-args]) != T_OBJECT ||
 	  !get_storage(ct=sp[7-args].u.object,image_colortable_program))
 	 Pike_error("Image.X.encode_truecolor_masks: illegal argument 8 (expected colortable object)\n");
  
-   if (sp[1-args].type!=T_INT)
+   if (TYPEOF(sp[1-args]) != T_INT)
       Pike_error("Image.X.encode_truecolor_masks: illegal argument 2 (expected integer)\n");
-   if (sp[2-args].type!=T_INT)
+   if (TYPEOF(sp[2-args]) != T_INT)
       Pike_error("Image.X.encode_truecolor_masks: illegal argument 3 (expected integer)\n");
 
-   if (sp[3-args].type!=T_INT)
+   if (TYPEOF(sp[3-args]) != T_INT)
       Pike_error("Image.X.encode_truecolor_masks: illegal argument 4 (expected integer)\n");
 
    image_x_examine_mask(sp+4-args,"argument 3 (red mask)",&rbits,&rshift);
@@ -530,7 +527,7 @@ static void image_x_encode_pseudocolor_1byte_exact(INT32 args,
       {
 	 if (translate) 
 	    { x=img->xsize; while (x--) *(d++)=translate[(*(s++))&mask]; }
-	 else MEMCPY(d,s,img->xsize),d+=img->xsize,s+=img->xsize;
+	 else memcpy(d,s,img->xsize),d+=img->xsize,s+=img->xsize;
 	 m=linemod;
 	 while (m--) *(d++)=0;
       }
@@ -606,7 +603,7 @@ static void image_x_encode_pseudocolor_1byte(INT32 args,
 	 x=img->xsize; 
 	 while (x--) 
 	 {
-	    b = ((*(s++))<<(32-bpp)) & 0xffffffff;
+	    b = (((unsigned long)*(s++))<<(32-bpp)) & 0xffffffff;
 	    bp = bpp;
 	    while (bp>8-bit)
 	    {
@@ -642,7 +639,7 @@ static void image_x_encode_pseudocolor_1byte(INT32 args,
    pop_n_elems(args);
    push_string(end_shared_string(dest2));
 }
-					     
+
 static void image_x_encode_pseudocolor_2byte(INT32 args,
 					     struct image *img,
 					     struct neo_colortable *nct,
@@ -728,37 +725,36 @@ static void image_x_encode_pseudocolor_2byte(INT32 args,
    pop_n_elems(args);
    push_string(end_shared_string(dest2));
 }
-					     
+
 void image_x_encode_pseudocolor(INT32 args)
 {
    INT32 bpp,alignbits,vbpp;
    struct image *img = NULL;
    struct neo_colortable *nct = NULL;
    char *translate=NULL;
-   
+
    if (args<5) 
       Pike_error("Image.X.encode_pseudocolor: too few arguments");
-   if (sp[1-args].type!=T_INT)
+   if (TYPEOF(sp[1-args]) != T_INT)
       Pike_error("Image.X.encode_pseudocolor: illegal argument 2 (expected integer)\n");
-   if (sp[2-args].type!=T_INT)
+   if (TYPEOF(sp[2-args]) != T_INT)
       Pike_error("Image.X.encode_pseudocolor: illegal argument 3 (expected integer)\n");
-   if (sp[3-args].type!=T_INT)
+   if (TYPEOF(sp[3-args]) != T_INT)
       Pike_error("Image.X.encode_pseudocolor: illegal argument 4 (expected integer)\n");
    bpp=sp[1-args].u.integer;
    alignbits=sp[2-args].u.integer;
    vbpp=sp[3-args].u.integer;
    if (!alignbits) alignbits=1;
 
-   if (sp[-args].type!=T_OBJECT ||
-       !(img=(struct image*)get_storage(sp[-args].u.object,image_program)))
+   if (TYPEOF(sp[-args]) != T_OBJECT ||
+       !(img=get_storage(sp[-args].u.object,image_program)))
       Pike_error("Image.X.encode_pseudocolor: illegal argument 1 (expected image object)\n");
-   if (sp[4-args].type!=T_OBJECT ||
-       !(nct=(struct neo_colortable*)
-	 get_storage(sp[4-args].u.object,image_colortable_program)))
+   if (TYPEOF(sp[4-args]) != T_OBJECT ||
+       !(nct=get_storage(sp[4-args].u.object,image_colortable_program)))
       Pike_error("Image.X.encode_pseudocolor: illegal argument 4 (expected colortable object)\n");
 
    if (args>5) {
-      if (sp[5-args].type!=T_STRING)
+      if (TYPEOF(sp[5-args]) != T_STRING)
 	 Pike_error("Image.X.encode_pseudocolor: illegal argument 6 (expected string)\n");
       else if (sp[5-args].u.string->len!=((vbpp>8)?2:1)<<vbpp)
 	 Pike_error("Image.X.encode_pseudocolor: illegal argument 6 "
@@ -769,7 +765,7 @@ void image_x_encode_pseudocolor(INT32 args)
 	 translate=sp[5-args].u.string->str;
    } 
    if ( vbpp==8 && bpp==8 && !((bpp*img->xsize)%alignbits) )
-      image_x_encode_pseudocolor_1byte_exact(args,img,nct,vbpp,bpp,alignbits,
+      image_x_encode_pseudocolor_1byte_exact(args,img,nct,bpp,vbpp,alignbits,
 					     (unsigned char*)translate);
    else if (vbpp<=8) 
       image_x_encode_pseudocolor_1byte(args,img,nct,bpp,vbpp,alignbits,
@@ -804,9 +800,10 @@ static void image_x_decode_truecolor(INT32 args)
 
    if (args<12) 
       Pike_error("Image.X.decode_truecolor: too few arguments\n");
-   if (sp[-args].type!=T_STRING) Pike_error("Image.X.decode_truecolor: illegal argument 1\n");
+   if (TYPEOF(sp[-args]) != T_STRING)
+      Pike_error("Image.X.decode_truecolor: illegal argument 1\n");
    for (i=1; i<12; i++)
-      if (sp[i-args].type!=T_INT) 
+      if (TYPEOF(sp[i-args]) != T_INT)
 	 Pike_error("Image.X.decode_truecolor: illegal argument %d\n",i+1);
 
    ps=sp[-args].u.string;
@@ -831,9 +828,8 @@ static void image_x_decode_truecolor(INT32 args)
 
    if (args>12)
    {
-      if (sp[12-args].type!=T_OBJECT ||
-	  !(nct=(struct neo_colortable*)
-	    get_storage(sp[12-args].u.object,image_colortable_program)))
+      if (TYPEOF(sp[12-args]) != T_OBJECT ||
+	  !(nct=get_storage(sp[12-args].u.object,image_colortable_program)))
 	 Pike_error("Image.X.decode_truecolor: illegal argument 13, expected colortable\n");
       if (nct->type!=NCT_FLAT)
 	 Pike_error("Image.X.decode_truecolor: illegal argument 13, expected colortable in flat mode\n");
@@ -862,7 +858,7 @@ static void image_x_decode_truecolor(INT32 args)
       push_int(width);
       push_int(height);
       o=clone_object(image_program,2);
-      img=(struct image*)get_storage(o,image_program);
+      img=get_storage(o,image_program);
 
       d=img->img;
       n=width*height;
@@ -908,7 +904,7 @@ static void image_x_decode_truecolor(INT32 args)
       push_int(width);
       push_int(height);
       o=clone_object(image_program,2);
-      img=(struct image*)get_storage(o,image_program);
+      img=get_storage(o,image_program);
 
       if (nct)
       {
@@ -983,20 +979,20 @@ void image_x_decode_truecolor_masks(INT32 args)
 
    if (args<9) 
       Pike_error("Image.X.decode_truecolor_masks: too few arguments (expected 7 arguments)\n");
-   if (sp[-args].type!=T_STRING)
+   if (TYPEOF(sp[-args]) != T_STRING)
       Pike_error("Image.X.decode_truecolor_masks: illegal argument 1 (expected image object)\n");
 
    if (args>9)
-      if (sp[9-args].type!=T_OBJECT ||
+       if (TYPEOF(sp[9-args]) != T_OBJECT ||
 	  !get_storage(ct=sp[9-args].u.object,image_colortable_program))
 	 Pike_error("Image.X.decode_truecolor_masks: illegal argument 8 (expected colortable object)\n");
  
-   if (sp[6-args].type!=T_INT)
+   if (TYPEOF(sp[6-args]) != T_INT)
       Pike_error("Image.X.decode_truecolor_masks: illegal argument 7 (expected integer)\n");
-   if (sp[7-args].type!=T_INT)
+   if (TYPEOF(sp[7-args]) != T_INT)
       Pike_error("Image.X.decode_truecolor_masks: illegal argument 8 (expected integer)\n");
 
-   if (sp[8-args].type!=T_INT)
+   if (TYPEOF(sp[8-args]) != T_INT)
       Pike_error("Image.X.decode_truecolor_masks: illegal argument 9 (expected integer)\n");
 
    image_x_examine_mask(sp+6-args,"argument 7 (red mask)",&rbits,&rshift);
@@ -1042,13 +1038,13 @@ void image_x_decode_pseudocolor(INT32 args)
 
    if (args<7) 
       Pike_error("Image.X.decode_pseudocolor: too few arguments\n");
-   if (sp[-args].type!=T_STRING) Pike_error("Image.X.decode_pseudocolor: illegal argument 1\n");
+   if (TYPEOF(sp[-args]) != T_STRING)
+      Pike_error("Image.X.decode_pseudocolor: illegal argument 1\n");
    for (i=1; i<6; i++)
-      if (sp[i-args].type!=T_INT) 
+      if (TYPEOF(sp[i-args]) != T_INT)
 	 Pike_error("Image.X.decode_pseudocolor: illegal argument %d\n",i+1);
-   if (sp[6-args].type!=T_OBJECT ||
-       !(nct=(struct neo_colortable*)
-	 get_storage(ncto=sp[6-args].u.object,image_colortable_program)))
+   if (TYPEOF(sp[6-args]) != T_OBJECT ||
+       !(nct=get_storage(ncto=sp[6-args].u.object,image_colortable_program)))
       Pike_error("Image.X.decode_pseudocolor: illegal argument 7\n");
 
    if (nct->type!=NCT_FLAT)
@@ -1076,7 +1072,7 @@ void image_x_decode_pseudocolor(INT32 args)
       push_int(width);
       push_int(height);
       o=clone_object(image_program,2);
-      img=(struct image*)get_storage(o,image_program);
+      img=get_storage(o,image_program);
 
       d=img->img;
       n=width*height;
@@ -1106,7 +1102,7 @@ void image_x_decode_pseudocolor(INT32 args)
       push_int(width);
       push_int(height);
       o=clone_object(image_program,2);
-      img=(struct image*)get_storage(o,image_program);
+      img=get_storage(o,image_program);
 
       d=img->img;
       m=height;
@@ -1153,14 +1149,14 @@ void image_x_encode_bitmap(INT32 args)
    struct image *img = NULL;
 
    if (!args)
-      SIMPLE_TOO_FEW_ARGS_ERROR("Image.X.encode_bitmap",1);
+      SIMPLE_TOO_FEW_ARGS_ERROR("encode_bitmap",1);
 
-   if (sp[-args].type!=T_OBJECT ||
-       !(img=(struct image*)get_storage(sp[-args].u.object,image_program)))
-      SIMPLE_BAD_ARG_ERROR("Image.X.encode_bitmap",1,"image object");
+   if (TYPEOF(sp[-args]) != T_OBJECT ||
+       !(img=get_storage(sp[-args].u.object,image_program)))
+      SIMPLE_BAD_ARG_ERROR("encode_bitmap",1,"Image.Image");
 
    if (!img->img)
-      SIMPLE_BAD_ARG_ERROR("Image.X.encode_bitmap",1,"image object with image");
+      SIMPLE_BAD_ARG_ERROR("encode_bitmap",1,"image object with image");
 
    xs=(img->xsize+7)>>3;
 
