@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 79c322fbe6c4d9f536631e37d1b7b2d5b1ae3014 $
 */
 
 /*
@@ -39,13 +38,13 @@ struct matrixX(_storage)
 
 static void matrixX(_mult)( INT32 args );
 
-static void Xmatrix(init_)(struct object *o)
+static void Xmatrix(init_)(struct object *UNUSED(o))
 {
    THIS->xsize=THIS->ysize=0;
    THIS->m=NULL;
 }
 
-static void Xmatrix(exit_)(struct object *o)
+static void Xmatrix(exit_)(struct object *UNUSED(o))
 {
    if (THIS->m) free(THIS->m);
 }
@@ -64,11 +63,11 @@ static void matrixX(_create)(INT32 args)
       bad_arg_error(PNAME, Pike_sp-args, args, 1, "", Pike_sp-args,
 		    "Has already been called.\n");
    
-   if (Pike_sp[-args].type==T_ARRAY)
+   if (TYPEOF(Pike_sp[-args]) == T_ARRAY)
    {
       ys=THIS->ysize=Pike_sp[-args].u.array->size;
 
-      if (ys<1 || Pike_sp[-args].u.array->item[0].type!=T_ARRAY)
+      if (ys<1 || TYPEOF(Pike_sp[-args].u.array->item[0]) != T_ARRAY)
       {
 	 push_svalue(Pike_sp-args);
 	 f_aggregate(THIS->ysize=ys=1); 
@@ -80,7 +79,7 @@ static void matrixX(_create)(INT32 args)
       for (i=0; i<ys; i++)
       {
 	struct array *a;
-	 if (Pike_sp[-args].u.array->item[i].type!=T_ARRAY)
+	 if (TYPEOF(Pike_sp[-args].u.array->item[i]) != T_ARRAY)
 	    SIMPLE_BAD_ARG_ERROR(PNAME,1,"array(array)");
 	 if (i==0) 
 	 {
@@ -98,7 +97,7 @@ static void matrixX(_create)(INT32 args)
 	 a = Pike_sp[-args].u.array->item[i].u.array;
 	 
 	 for (j=0; j<xs; j++)
-	    switch (a->item[j].type)
+	    switch (TYPEOF(a->item[j]))
 	    {
 	       case T_INT:
 		  *(m++) = (FTYPE)a->item[j].u.integer;
@@ -109,33 +108,27 @@ static void matrixX(_create)(INT32 args)
 	      case T_OBJECT:
 		{
 		  INT64 x;
-#ifdef AUTO_BIGNUM
 		  if (a->item[j].u.object->prog != get_auto_bignum_program()) {
-#endif
 		    /* Use push_svalue() so that we support subtypes... */
 		    push_svalue(a->item+j);
 		    o_cast_to_int();
-		    if (Pike_sp[-1].type == T_INT) {
+		    if (TYPEOF(Pike_sp[-1]) == T_INT) {
 		      *(m++) = (FTYPE)Pike_sp[-1].u.integer;
 		      pop_stack();
 		      break;
-#ifdef AUTO_BIGNUM
-		    } else if ((Pike_sp[-1].type == T_OBJECT) &&
+		    } else if ((TYPEOF(Pike_sp[-1]) == T_OBJECT) &&
 			       (Pike_sp[-1].u.object->prog ==
 				get_auto_bignum_program()) &&
 			       int64_from_bignum(&x, Pike_sp[-1].u.object)) {
 		      *(m++) = (FTYPE)x;
 		      pop_stack();
 		      break;
-#endif
 		    }
 		    pop_stack();
-#ifdef AUTO_BIGNUM
 		  } else if (int64_from_bignum(&x, a->item[j].u.object)) {
 		    *(m++) = (FTYPE)x;
 		    break;
 		  }
-#endif
 		}
 		/* FALL_THROUGH */
 	      default:
@@ -145,36 +138,32 @@ static void matrixX(_create)(INT32 args)
       }
       THIS->xsize=xs;
    }
-   else if (Pike_sp[-args].type==T_INT)
+   else if (TYPEOF(Pike_sp[-args]) == T_INT)
    {
       FTYPE z = DO_NOT_WARN((FTYPE)0.0);
 
       if (args<2)
 	 SIMPLE_TOO_FEW_ARGS_ERROR(PNAME,2);
-      if (Pike_sp[1-args].type!=T_INT)
+      if (TYPEOF(Pike_sp[1-args]) != T_INT)
 	 SIMPLE_BAD_ARG_ERROR(PNAME,2,"int");
 
       if ((THIS->xsize=xs=Pike_sp[-args].u.integer)<=0)
-	 SIMPLE_BAD_ARG_ERROR(PNAME,1,"int > 0");
+	 SIMPLE_BAD_ARG_ERROR(PNAME,1,"int(1..)");
       if ((THIS->ysize=ys=Pike_sp[1-args].u.integer)<=0)
-	 SIMPLE_BAD_ARG_ERROR(PNAME,2,"int > 0");
+	 SIMPLE_BAD_ARG_ERROR(PNAME,2,"int(1..)");
+
+      THIS->m=m=xcalloc(sizeof(FTYPE),xs*ys);
 
-      THIS->m=m=malloc(sizeof(FTYPE)*xs*ys);
-      if (!m)
-	 SIMPLE_OUT_OF_MEMORY_ERROR(PNAME,
-				    sizeof(FTYPE)*xs*ys);
-      
       if (args>2) {
-	 if (Pike_sp[2-args].type==T_INT)
+	 if (TYPEOF(Pike_sp[2-args]) == T_INT)
 	    z=(FTYPE)Pike_sp[2-args].u.integer;
-	 else if (Pike_sp[2-args].type==T_FLOAT)
+	 else if (TYPEOF(Pike_sp[2-args]) == T_FLOAT)
 	    z=(FTYPE)Pike_sp[2-args].u.float_number;
-	 else if (Pike_sp[2-args].type==T_STRING)
+	 else if (TYPEOF(Pike_sp[2-args]) == T_STRING)
 	 {
 	    if (Pike_sp[2-args].u.string==s__clr)
 	    {
 	       /* internal call: don't care */
-	       MEMSET(m,0,xs*ys*sizeof(FTYPE));
 	       goto done_made;
 	    }
 	    else if (Pike_sp[2-args].u.string==s_identity)
@@ -204,7 +193,7 @@ static void matrixX(_create)(INT32 args)
 done_made:
       ;
    }
-   else if (Pike_sp[-args].type==T_STRING)
+   else if (TYPEOF(Pike_sp[-args]) == T_STRING)
    {
       char *dummy;
       INT_TYPE side,n;
@@ -232,9 +221,8 @@ done_made:
 
 	 /* "rotate",size,degrees,x,y,z */
 
-	 if (args>3 && Pike_sp[3-args].type==T_OBJECT &&
-	     ((mx=(struct matrixX(_storage)*)
-	       get_storage(Pike_sp[-1].u.object,XmatrixY(math_,_program)))))
+	 if (args>3 && TYPEOF(Pike_sp[3-args]) == T_OBJECT &&
+	     ((mx=get_storage(Pike_sp[-1].u.object,XmatrixY(math_,_program)))))
 	 {
 	    if (mx->xsize*mx->ysize!=3)
 	       SIMPLE_BAD_ARG_ERROR(PNAME,4,"Matrix of size 1x3 or 3x1");
@@ -296,29 +284,30 @@ void matrixX(_cast)(INT32 args)
       push_int(0);
    }
 
-   if (args)
-      if (Pike_sp[-1].type==T_STRING) {
-	 if (Pike_sp[-1].u.string==s_array)
-	 {
-	    int i,j;
-	    int xs=THIS->xsize,ys=THIS->ysize;
-	    FTYPE *m=THIS->m;
-	    check_stack(DO_NOT_WARN((long)(xs+ys)));
-	    pop_n_elems(args);
-	    for (i=0; i<ys; i++)
-	    {
-	      for (j=0; j<xs; j++)
-		PUSH_ELEM(*(m++));
-	      f_aggregate(xs);
-	    }
-	    f_aggregate(ys);
-	    return;
-	 }
-         else
-           Pike_error("Can only cast to array.\n");
-      }
+   if( !args || TYPEOF(Pike_sp[-1]) != T_STRING )
+     SIMPLE_BAD_ARG_ERROR("cast",1,"string");
 
-   SIMPLE_BAD_ARG_ERROR("cast",1,"string");
+   if( Pike_sp[-1].u.string != literal_array_string )
+   {
+     pop_n_elems(args);
+     push_undefined();
+     return;
+   }
+
+   {
+     int i,j;
+     int xs=THIS->xsize,ys=THIS->ysize;
+     FTYPE *m=THIS->m;
+     check_stack(DO_NOT_WARN((long)(xs+ys)));
+     pop_n_elems(args);
+     for (i=0; i<ys; i++)
+     {
+       for (j=0; j<xs; j++)
+         PUSH_ELEM(*(m++));
+       f_aggregate(xs);
+     }
+     f_aggregate(ys);
+   }
 }
 
 
@@ -366,7 +355,7 @@ void matrixX(__sprintf)(INT32 args)
 	    return;
 	 }
 
-	 push_constant_text("Math." PNAME "( ({ ({ ");
+	 push_text("Math." PNAME "( ({ ({ ");
 	 n=1;
 	 for (y=0; y<THIS->ysize; y++)
 	 {
@@ -377,10 +366,10 @@ void matrixX(__sprintf)(INT32 args)
 	       push_text(buf); n++;
 	    }
 	    if (y<THIS->ysize-1)
-	       push_constant_text("}),\n                ({ "); 
+	       push_text("}),\n                ({ "); 
 	    n++;
 	 }
-	 push_constant_text("}) }) )"); 
+	 push_text("}) }) )"); 
 	 f_add(n);
 	 stack_pop_n_elems_keep_top(args);
 	 return;
@@ -511,9 +500,8 @@ static void matrixX(_add)(INT32 args)
       return;
    }
 
-   if (Pike_sp[-1].type!=T_OBJECT ||
-       !((mx=(struct matrixX(_storage)*)
-	  get_storage(Pike_sp[-1].u.object,XmatrixY(math_,_program)))))
+   if (TYPEOF(Pike_sp[-1]) != T_OBJECT ||
+       !((mx=get_storage(Pike_sp[-1].u.object,XmatrixY(math_,_program)))))
       SIMPLE_BAD_ARG_ERROR("`+",1,"object(Math.Matrix)");
 
    if (mx->xsize != THIS->xsize || mx->ysize != THIS->ysize)
@@ -555,9 +543,8 @@ static void matrixX(_sub)(INT32 args)
 	 return;
       }
 
-      if (Pike_sp[-1].type!=T_OBJECT ||
-	  !((mx=(struct matrixX(_storage)*)
-	     get_storage(Pike_sp[-1].u.object,XmatrixY(math_,_program)))))
+      if (TYPEOF(Pike_sp[-1]) != T_OBJECT ||
+	  !((mx=get_storage(Pike_sp[-1].u.object,XmatrixY(math_,_program)))))
 	 SIMPLE_BAD_ARG_ERROR("`-",1,"object(Math.Matrix)");
 
       if (mx->xsize != THIS->xsize ||
@@ -666,12 +653,12 @@ static void matrixX(_mult)(INT32 args)
       return;
    }
 
-   if (Pike_sp[-1].type==T_INT)
+   if (TYPEOF(Pike_sp[-1]) == T_INT)
    {
       z=(FTYPE)Pike_sp[-1].u.integer;
       goto scalar_mult;
    }
-   else if (Pike_sp[-1].type==T_FLOAT)
+   else if (TYPEOF(Pike_sp[-1]) == T_FLOAT)
    {
       z=(FTYPE)Pike_sp[-1].u.float_number;
 scalar_mult:
@@ -689,9 +676,8 @@ scalar_mult:
       return;
    }
 	 
-   if (Pike_sp[-1].type!=T_OBJECT ||
-       !((mx=(struct matrixX(_storage)*)
-	  get_storage(Pike_sp[-1].u.object,XmatrixY(math_,_program)))))
+   if (TYPEOF(Pike_sp[-1]) != T_OBJECT ||
+       !((mx=get_storage(Pike_sp[-1].u.object,XmatrixY(math_,_program)))))
       SIMPLE_BAD_ARG_ERROR("`*",1,"object(Math.Matrix)");
 
    if (mx->xsize != THIS->ysize)
@@ -733,9 +719,8 @@ static void matrixX(_cross)(INT32 args)
 
    pop_n_elems(args-1); /* shouldn't be needed */
 
-   if (Pike_sp[-1].type!=T_OBJECT ||
-       !((mx=(struct matrixX(_storage)*)
-	  get_storage(Pike_sp[-1].u.object,XmatrixY(math_,_program)))))
+   if (TYPEOF(Pike_sp[-1]) != T_OBJECT ||
+       !((mx=get_storage(Pike_sp[-1].u.object,XmatrixY(math_,_program)))))
       SIMPLE_BAD_ARG_ERROR("cross",1,"object(Math.Matrix)");
 
    if (mx->xsize*mx->ysize != 3 ||
@@ -768,9 +753,8 @@ static void matrixX(_dot)(INT32 args)
   
   pop_n_elems(args-1); 
   
-  if (Pike_sp[-1].type!=T_OBJECT ||
-      !((mx=(struct matrixX(_storage)*)
-	 get_storage(Pike_sp[-1].u.object,XmatrixY(math_,_program)))))
+  if (TYPEOF(Pike_sp[-1]) != T_OBJECT ||
+      !((mx=get_storage(Pike_sp[-1].u.object,XmatrixY(math_,_program)))))
     SIMPLE_BAD_ARG_ERROR("dot_product",1,"object(Math.Matrix)");
   
   if(!(mx->xsize==THIS->xsize &&
@@ -805,9 +789,8 @@ static void matrixX(_convolve)(INT32 args)
    if (args<1)
       SIMPLE_TOO_FEW_ARGS_ERROR("convolve",1);
 
-   if (Pike_sp[-args].type!=T_OBJECT ||
-       !((bmx=(struct matrixX(_storage)*)
-	  get_storage(Pike_sp[-args].u.object,XmatrixY(math_,_program)))))
+   if (TYPEOF(Pike_sp[-args]) != T_OBJECT ||
+       !((bmx=get_storage(Pike_sp[-args].u.object,XmatrixY(math_,_program)))))
       SIMPLE_BAD_ARG_ERROR("convolve",1,"object(Math.Matrix)");
 
    if (bmx->xsize==0 || bmx->ysize==0 ||
@@ -974,8 +957,6 @@ static void matrixX(_ysize)(INT32 args)
 void Xmatrix(init_math_)(void)
 {
 #define MKSTR(X) make_shared_binary_string(X,CONSTANT_STRLEN(X))
-  if( !s_array )
-    s_array=MKSTR("array");
   if( !s_rotate )
     s_rotate=MKSTR("rotate");
   if( !s__clr )
@@ -995,7 +976,7 @@ void Xmatrix(init_math_)(void)
 		      tFunc(tInt1Plus tInt1Plus tOr4(tInt,tFloat,tString,tVoid), tVoid)), ID_PROTECTED);
 
    ADD_FUNCTION("cast",matrixX(_cast),
-		tFunc(tStr, tArr(tArr(tFloat))), 0);
+		tFunc(tStr, tArr(tArr(tFloat))), ID_PROTECTED);
    ADD_FUNCTION("vect",matrixX(_vect), tFunc(tNone,tArr(PTYPE)), 0);
    ADD_FUNCTION("_sprintf",matrixX(__sprintf), tFunc(tInt tMapping, tStr), 0);
 
@@ -1022,18 +1003,11 @@ void Xmatrix(init_math_)(void)
    ADD_FUNCTION("``*",matrixX(_mult),
 		tFunc(tOr3(tObj,tFloat,tInt), tObj), 0);
 
-   ADD_FUNCTION("`",matrixX(_dot),
-		tFunc(tOr3(tObj,tFloat,tInt), tObj), 0);
-   ADD_FUNCTION("``",matrixX(_dot),
-		tFunc(tOr3(tObj,tFloat,tInt), tObj), 0);
-
    ADD_FUNCTION("dot_product",matrixX(_dot), tFunc(tObj, tObj), 0);
 
    ADD_FUNCTION("convolve",matrixX(_convolve), tFunc(tObj, tObj), 0);
    
    ADD_FUNCTION("cross",matrixX(_cross), tFunc(tObj, tObj), 0);
-   ADD_FUNCTION("`",matrixX(_cross), tFunc(tObj, tObj), 0);
-   ADD_FUNCTION("``",matrixX(_cross), tFunc(tObj, tObj), 0);
 
    ADD_FUNCTION("xsize", matrixX(_xsize), tFunc(tNone, tInt), 0);
    ADD_FUNCTION("ysize", matrixX(_ysize), tFunc(tNone, tInt), 0);
@@ -1045,10 +1019,6 @@ void Xmatrix(init_math_)(void)
 
 void Xmatrix(exit_math_)(void)
 {
-  if (s_array) {
-    free_string(s_array);
-    s_array = NULL;
-  }
   if (s_rotate) {
     free_string(s_rotate);
     s_rotate = NULL;
