@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: a85c2f9d9ac4627814210b43103becdadf20e300 $
 */
 
 /*
@@ -60,12 +59,15 @@
 #endif
 #endif
 
-#ifdef HAVE_FFMPEG_AVCODEC_H
-#include <ffmpeg/avcodec.h>
-#else
+#define FF_API_AUDIO_OLD 1
+#define FF_API_VIDEO_OLD 1
+
 #ifdef HAVE_LIBAVCODEC_AVCODEC_H
 #include <libavcodec/avcodec.h>
 #else
+#ifdef HAVE_FFMPEG_AVCODEC_H
+#include <ffmpeg/avcodec.h>
+#else
 #ifdef HAVE_LIBFFMPEG_AVCODEC_H
 #include <libffmpeg/avcodec.h>
 #else
@@ -76,6 +78,17 @@
 #endif
 #endif
 
+#ifdef HAVE_LIBAVUTIL_AVUTIL_H
+#include <libavutil/avutil.h>
+/* FIXME: The following test is an approximation.
+ *        50.7.0 is known not to have the enum.
+ *        50.43.0 is known to have it.
+ */
+#if (LIBAVUTIL_VERSION_MAJOR > 50) || (LIBAVUTIL_VERSION_MINOR > 7)
+#define USE_AVMEDIA_TYPE_ENUM 1
+#endif
+#endif
+
 #ifdef AV_MEM
 #define FF_ALLOC(x)	av_mallocz(x)
 #define FF_FREE(x)	av_free(x)
@@ -102,7 +115,13 @@ typedef struct {
 int encoder_flg(AVCodec *codec) {
   int flg = -1;
 
-  if( codec->encode )
+  if( codec->
+#ifdef HAVE_AVCODEC_ENCODE2
+      encode2
+#else
+      encode
+#endif
+      )
     flg = 1;
   else if( codec->decode )
     flg = 0;
@@ -141,29 +160,29 @@ static void f_create(INT32 args)
 
   switch(args) {
     case 5:
-      if(Pike_sp[-1].type != T_INT)
+      if(TYPEOF(Pike_sp[-1]) != T_INT)
 	Pike_error("Invalid argument 5, expected int.\n");
       chns = (u_short)Pike_sp[-1].u.integer;
       Pike_sp--;
     case 4:
-      if(Pike_sp[-1].type != T_INT)
+      if(TYPEOF(Pike_sp[-1]) != T_INT)
 	Pike_error("Invalid argument 4, expected int.\n");
       wide = (u_short)Pike_sp[-1].u.integer;
       Pike_sp--;
     case 3:
-      if(Pike_sp[-1].type != T_INT)
+      if(TYPEOF(Pike_sp[-1]) != T_INT)
 	Pike_error("Invalid argument 3, expected int.\n");
       rate = (u_short)Pike_sp[-1].u.integer;
       Pike_sp--;
 
     case 2:
-      if(Pike_sp[-1].type != T_INT)
+      if(TYPEOF(Pike_sp[-1]) != T_INT)
 	Pike_error("Invalid argument 2, expected int.\n");
       THIS->encoder = (u_short)Pike_sp[-1].u.integer;
       Pike_sp--;
 
     /* case 1: */
-      if(Pike_sp[-1].type != T_INT)
+      if(TYPEOF(Pike_sp[-1]) != T_INT)
 	Pike_error("Invalid argument 1, expected int.\n");
       codec_id = (u_short)Pike_sp[-1].u.integer;
       Pike_sp--;
@@ -178,6 +197,7 @@ static void f_create(INT32 args)
       }
 
       memset(&THIS->codec_context, 0, sizeof(THIS->codec_context));
+      /* FIXME: Use avcodec_open2() if available. */
       if (avcodec_open(&THIS->codec_context, codec) < 0)
         Pike_error("Could not open codec.\n");
       THIS->codec = codec;
@@ -209,7 +229,7 @@ static void f_codec_info(INT32 args) {
   pop_n_elems(args);
   if(THIS->codec) {
     push_text("name");		push_text( THIS->codec->name );
-    push_text("type");		push_int( THIS->codec->type );
+    ref_push_string(literal_type_string);		push_int( THIS->codec->type );
     push_text("id");		push_int( THIS->codec->id );
     push_text("encoder_flg");	push_int( encoder_flg(THIS->codec) );
     f_aggregate_mapping( 2*4 );
@@ -236,13 +256,13 @@ static void f_set_codec_param(INT32 args) {
 
   if(args != 2)
     Pike_error("Invalid number of arguments to set_codec_param().\n");
-  if(Pike_sp[-args].type != T_STRING)
+  if(TYPEOF(Pike_sp[-args]) != T_STRING)
     Pike_error("Invalid argument 1, expected string.\n");
   pname = Pike_sp[-args].u.string;
 
   /* bit_rate */
   if(!strncmp(pname->str, "bit_rate", 8)) {
-    if(Pike_sp[-1].type != T_INT)
+    if(TYPEOF(Pike_sp[-1]) != T_INT)
       Pike_error("Invalid argument 2, expected integer.\n");
     /* FIXME: test correct value of bit rate argument */
     THIS->codec_context.bit_rate = Pike_sp[-1].u.integer;
@@ -253,7 +273,7 @@ static void f_set_codec_param(INT32 args) {
 
   /* sample_rate */
   if(!strncmp(pname->str, "sample_rate", 11)) {
-    if(Pike_sp[-1].type != T_INT)
+    if(TYPEOF(Pike_sp[-1]) != T_INT)
       Pike_error("Invalid argument 2, expected integer.\n");
     /* FIXME: test correct value of bit rate argument */
     THIS->codec_context.sample_rate = Pike_sp[-1].u.integer;
@@ -264,7 +284,7 @@ static void f_set_codec_param(INT32 args) {
 
   /* channels */
   if(!strncmp(pname->str, "channels", 8)) {
-    if(Pike_sp[-1].type != T_INT)
+    if(TYPEOF(Pike_sp[-1]) != T_INT)
       Pike_error("Invalid argument 2, expected integer.\n");
     /* FIXME: test correct value of bit rate argument */
     THIS->codec_context.channels = (u_short)Pike_sp[-1].u.integer;
@@ -294,20 +314,28 @@ static void f_get_codec_status(INT32 args) {
   }
   
   push_text("name");		push_text( THIS->codec->name );
-  push_text("type");		push_int( THIS->codec->type );
+  ref_push_string(literal_type_string);		push_int( THIS->codec->type );
   push_text("id");		push_int( THIS->codec->id );
   push_text("encoder_flg");	push_int( encoder_flg(THIS->codec) );
   push_text("flags");		push_int( THIS->codec_context.flags );
   cnt = 5;
 
+#ifdef USE_AVMEDIA_TYPE_ENUM
+  if(THIS->codec->type == AVMEDIA_TYPE_AUDIO) {
+#else
   if(THIS->codec->type == CODEC_TYPE_AUDIO) {
+#endif
     /* audio only */
     push_text("sample_rate");	push_int( THIS->codec_context.sample_rate );
     push_text("channels");	push_int( THIS->codec_context.channels );
     cnt += 2;
   }
 
+#ifdef USE_AVMEDIA_TYPE_ENUM
+  if(THIS->codec->type == AVMEDIA_TYPE_VIDEO) {
+#else
   if(THIS->codec->type == CODEC_TYPE_VIDEO) {
+#endif
     /* video only */
     push_text("frame_rate");
 #ifdef HAVE_AVCODECCONTEXT_FRAME_RATE
@@ -325,6 +353,17 @@ static void f_get_codec_status(INT32 args) {
   f_aggregate_mapping(2 * cnt);
 }
 
+#ifndef HAVE_AVCODEC_DECODE_AUDIO
+#define avcodec_decode_audio avcodec_decode_audio2
+#ifndef HAVE_AVCODEC_DECODE_AUDIO2
+#define avcodec_decode_audio2 avcodec_decode_audio3
+#define USE_AVPACKET_FOR_DECODE
+#ifndef HAVE_AVCODEC_DECODE_AUDIO3
+  /* FIXME: Support avcodec_decode_audio4(). */
+#endif
+#endif
+#endif
+
 /*! @decl mapping|int decode(string data)
  *!
  *! Returns a mapping with the new decoded frame and lenght of
@@ -349,8 +388,11 @@ static void f_decode(INT32 args) {
   struct pike_string *idata;
   int len, samples_size;
   struct svalue feeder;
+#ifdef USE_AVPACKET_FOR_DECODE
+  AVPacket avp;
+#endif
 
-  if(Pike_sp[-args].type != T_STRING)
+  if(TYPEOF(Pike_sp[-args]) != T_STRING)
     Pike_error("Invalid argument 1, expected string.\n");
 
   idata = Pike_sp[-args].u.string;
@@ -370,7 +412,7 @@ static void f_decode(INT32 args) {
   if(args > 1) {
     /* FIXME: shuffler part not implemented, yet */
 #if NOT_IMPLEMENTED_YET
-    if(Pike_sp[-1].type != T_FUNCTION)
+    if(TYPEOF(Pike_sp[-1]) != T_FUNCTION)
       Pike_error("Invalid argument 2, expected function.\n");
     feeder = Pike_sp[1-args].u.svalue;
     apply_svalue(&feeder, 0); /* we want more data */
@@ -381,9 +423,18 @@ static void f_decode(INT32 args) {
   }
 
   /* one pass decoding */
+#ifdef USE_AVPACKET_FOR_DECODE
+  av_init_packet(&avp);
+  avp.data = STR0(idata);
+  avp.size = idata->len;
+  len = avcodec_decode_audio(&THIS->codec_context,
+			     (short *)THIS->outbuf, &samples_size,
+			     &avp);
+#else
   len = avcodec_decode_audio(&THIS->codec_context,
 			     (short *)THIS->outbuf, &samples_size,
 		  	     STR0(idata), idata->len);
+#endif
   if(len < 0)
     Pike_error("Error while decoding.\n");
   if(samples_size > 0) {
@@ -432,10 +483,20 @@ static void f_list_codecs(INT32 args) {
   int cnt = 0;
   AVCodec *codec;
 
-  avcodec_init();
   avcodec_register_all();
 
   pop_n_elems(args);
+#ifdef HAVE_AV_CODEC_NEXT
+  codec = NULL;
+  while ((codec = av_codec_next(codec))) {
+    cnt++;
+    push_text("name");		push_text( codec->name );
+    ref_push_string(literal_type_string);		push_int( codec->type );
+    push_text("id");		push_int( codec->id );
+    push_text("encoder_flg");	push_int( encoder_flg(codec) );
+    f_aggregate_mapping( 2*4 );
+  }
+#else /* !HAVE_AV_CODEC_NEXT */
   if(first_avcodec == NULL) {
     push_int(0);
     return;
@@ -445,12 +506,13 @@ static void f_list_codecs(INT32 args) {
   while(codec != NULL) {
     cnt++;
     push_text("name");		push_text( codec->name );
-    push_text("type");		push_int( codec->type );
+    ref_push_string(literal_type_string);		push_int( codec->type );
     push_text("id");		push_int( codec->id );
     push_text("encoder_flg");	push_int( encoder_flg(codec) );
     codec = codec->next;
     f_aggregate_mapping( 2*4 );
   }
+#endif /* HAVE_AV_CODEC_NEXT */
   push_array(aggregate_array( cnt ));
 }
 
@@ -509,7 +571,6 @@ static void init_ffmpeg_data(struct object *obj) {
   THIS->outbuf = NULL;
   THIS->encoder = 0;
 
-  avcodec_init();
   avcodec_register_all(); /* FIXME: register only "interesting" codec ? */
 }
 
@@ -625,6 +686,15 @@ PIKE_MODULE_INIT {
    * Internal type of codec.
    *
    */
+#ifdef USE_AVMEDIA_TYPE_ENUM
+  add_integer_constant("AVMEDIA_TYPE_UNKNOWN", AVMEDIA_TYPE_UNKNOWN, 0);
+  add_integer_constant("AVMEDIA_TYPE_AUDIO", AVMEDIA_TYPE_AUDIO, 0);
+  add_integer_constant("AVMEDIA_TYPE_VIDEO", AVMEDIA_TYPE_VIDEO, 0);
+  add_integer_constant("AVMEDIA_TYPE_DATA", AVMEDIA_TYPE_DATA, 0);
+  add_integer_constant("AVMEDIA_TYPE_SUBTITLE", AVMEDIA_TYPE_SUBTITLE, 0);
+  add_integer_constant("AVMEDIA_TYPE_ATTACHMENT", AVMEDIA_TYPE_ATTACHMENT, 0);
+  add_integer_constant("AVMEDIA_TYPE_NB", AVMEDIA_TYPE_NB, 0);
+#else
   add_integer_constant("CODEC_TYPE_AUDIO", CODEC_TYPE_AUDIO, 0);
   add_integer_constant("CODEC_TYPE_VIDEO", CODEC_TYPE_VIDEO, 0);
 #ifndef CODEC_ID_NONE
@@ -634,6 +704,7 @@ PIKE_MODULE_INIT {
 #ifdef CODEC_TYPE_UNKNOWN
   add_integer_constant("CODEC_TYPE_UNKNOWN", CODEC_TYPE_UNKNOWN, 0);
 #endif
+#endif
 #endif
 
   start_new_program();
