@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: c5ed76403644cbbdf9c155d5539e74446d3027fe $
 */
 
 #include "global.h"
@@ -31,6 +30,7 @@
 #include "dynamic_buffer.h"
 #include "operators.h"
 #include "builtin_functions.h"
+#include "pike_types.h"
 
 #if defined(USE_GMP) || defined(USE_GMP2)
 
@@ -398,11 +398,11 @@ static void image_xface_decode(INT32 args)
   struct object *o;
   struct image *img;
 
-  if(args<1 || sp[-args].type!=T_STRING)
+  if(args<1 || TYPEOF(sp[-args]) != T_STRING)
     Pike_error("Image.XFace.decode: Illegal arguments\n");
 
   o=clone_object(image_program,0);
-  img=(struct image*)get_storage(o,image_program);
+  img=get_storage(o,image_program);
   if (!img) Pike_error("image no image? foo?\n"); /* should never happen */
   img->img=malloc(sizeof(rgb_group)*48*48);
   if (!img->img) {
@@ -436,10 +436,9 @@ static void image_xface_encode(INT32 args)
   struct pike_string *res;
 
   if (args<1 
-      || sp[-args].type!=T_OBJECT
-      || !(img=(struct image*)
-	   get_storage(sp[-args].u.object,image_program))
-      || (args>1 && sp[1-args].type!=T_MAPPING))
+      || TYPEOF(sp[-args]) != T_OBJECT
+      || !(img=get_storage(sp[-args].u.object,image_program))
+      || (args>1 && TYPEOF(sp[1-args]) != T_MAPPING))
     Pike_error("Image.XFace.encode: Illegal arguments\n");
   
   if (!img->img)
@@ -483,12 +482,12 @@ static void image_xface_encode(INT32 args)
 
 static void image_xface_decode_header(INT32 args)
 {
-  if(args<1 || sp[-args].type!=T_STRING)
-    Pike_error("Image.XFace.decode: Illegal arguments\n");
+  if(args<1 || TYPEOF(sp[-args]) != T_STRING)
+    Pike_error("Image.XFace.decode_header: Illegal arguments\n");
 
   pop_n_elems(args);
 
-  push_text("type"); 
+  ref_push_string(literal_type_string);
   push_text("image/x-xface");
 
   push_text("xsize");
@@ -516,7 +515,7 @@ PIKE_MODULE_INIT
 #ifdef DYNAMIC_MODULE
    push_text("Image.Image");
    SAFE_APPLY_MASTER("resolv",1);
-   if (sp[-1].type==T_PROGRAM)
+   if (TYPEOF(sp[-1]) == T_PROGRAM)
       image_program=program_from_svalue(sp-1);
    pop_stack();
 #endif /* DYNAMIC_MODULE */
