@@ -377,10 +377,13 @@ static void f_resultset_dup( INT32 args )
   struct object *o = clone_object( resultset_program, 0 );
   if(THIS->d)
   {
-    ResultSet *d = malloc( THIS->d->num_docs * 8 + 4 );
-    memcpy( d, THIS->d, THIS->d->num_docs * 8 + 4 );
-    T(o)->d = d;
-    T(o)->allocated_size = T(o)->d->num_docs;
+    if (T(o)->allocated_size < THIS->d->num_docs) {
+      ResultSet *d = xalloc( THIS->d->num_docs * 8 + 4 );
+      if (T(o)->d) free(T(o)->d);
+      T(o)->d = d;
+      T(o)->allocated_size = T(o)->d->num_docs;
+    }
+    memcpy( T(o)->d, THIS->d, THIS->d->num_docs * 8 + 4 );
   }
   pop_n_elems( args );
   wf_resultset_push( o );
@@ -420,6 +423,10 @@ static void f_resultset_overhead( INT32 args )
 static void duplicate_resultset( struct object *dest,
 				 struct object *src )
 {
+  if (T(dest)->d) {
+    free(T(dest)->d);
+    T(dest)->d = NULL;
+  }
   if( src->refs == 1 )
   {
     /* Destructively move the data. */
@@ -433,7 +440,7 @@ static void duplicate_resultset( struct object *dest,
     /* Ok, we have to copy it. */
     int size = 4+4*T(src)->allocated_size*2;
     T(dest)->allocated_size = T(src)->allocated_size;
-    T(dest)->d              = malloc( size );
+    T(dest)->d              = xalloc( size );
     memcpy( T(dest)->d, T(src)->d, size );
   }
 }
@@ -453,7 +460,7 @@ static void f_resultset_or( INT32 args )
   struct object *right;
   int lp=-1, rp=-1;
 
-  int left_used=1, right_used=1;
+  int left_valid = 0, right_valid = 0;
   int left_left=1, right_left=1;
   int right_size, left_size;
 
@@ -464,8 +471,7 @@ static void f_resultset_or( INT32 args )
 
   set_r = T( WF_RESULTSET(right) )->d;
 
-
-  if( !set_l )
+  if( !set_l || !set_l->num_docs )
   {
     if( set_r )
       duplicate_resultset( res, right );
@@ -474,7 +480,7 @@ static void f_resultset_or( INT32 args )
     return;
   }
 
-  if( !set_r )
+  if( !set_r || !set_r->num_docs )
   {
     duplicate_resultset( res, left );
     pop_n_elems(args);
@@ -482,74 +488,63 @@ static void f_resultset_or( INT32 args )
     return;
   }
 
-  left_size = set_l->num_docs;
-  right_size = set_r->num_docs;
-  
+  left_left = !!(left_size = set_l->num_docs);
+  right_left = !!(right_size = set_r->num_docs);
+
   while( left_left || right_left )
   {
-    if( left_left && left_used ) /* New from left */
+    if( left_left && !left_valid ) /* New from left */
     {
-      if( ++lp == left_size )
+      if( ++lp >= left_size )
       {
 	left_left = 0;
 	left_rank = 0;
+	lp = left_size;
 	if( !right_left )
-	  continue;
+	  break;
       }
       else
       {
 	left_doc = set_l->hits[lp].doc_id;
 	left_rank = set_l->hits[lp].ranking;
-	left_used = 0;
-	if( !left_rank ) left_rank = right_rank;
-	if( !right_left )right_rank = left_rank;
+	left_valid = 1;
       }
     }
 
-    if( right_left && right_used ) /* New from right */
+    if( right_left && !right_valid ) /* New from right */
     {
-      if( ++rp == right_size )
+      if( ++rp >= right_size )
       {
 	right_left = 0;
 	right_rank = 0;
+	rp = right_size;
 	if( !left_left )
-	  continue;
+	  break;
       }
       else
       {
 	right_doc = set_r->hits[rp].doc_id;
 	right_rank = set_r->hits[rp].ranking;
-	right_used = 0;
-	if( !left_rank ) left_rank = right_rank;
-	if( !right_left )right_rank = left_rank;
+	right_valid = 1;
       }
     }
 
-
-    if(!right_left || (left_doc <= right_doc))
+    if(left_valid && ((left_doc <= right_doc) || !right_valid))
     {
       if(left_doc>last)
 	wf_resultset_add( res, (last = left_doc), left_rank );
       else if( left_doc == last )
 	wf_resultset_add_ranking( res, -1, left_rank );
-      left_used=1;
-    }
-
-    if(!left_left || (right_doc <= left_doc ) )
+      left_valid = 0;
+    } else if(right_valid && (right_doc <= left_doc || !left_valid ) )
     {
       if(right_doc>last)
 	wf_resultset_add( res, (last = right_doc), right_rank );
       else if( right_doc == last )
 	wf_resultset_add_ranking( res, -1, right_rank );
-      right_used=1;
+      right_valid = 0;
     }
   }
-  if( !left_used )
-    if(left_doc!=last)
-      wf_resultset_add( res, (last = left_doc), left_rank );
-  if( !right_used )
-    if(right_doc!=last)
-      wf_resultset_add( res, (last = right_doc), right_rank );
   pop_n_elems( args );
   wf_resultset_push( res );
 }
@@ -568,7 +563,7 @@ static void f_resultset_intersect( INT32 args )
   struct object *right;
   int lp=-1, rp=-1;
 
-  int left_used=1, right_used=1;
+  int left_valid = 0, right_valid = 0;
   int left_left=1, right_left=1;
   int right_size, left_size;
 
@@ -580,21 +575,22 @@ static void f_resultset_intersect( INT32 args )
   right = WF_RESULTSET( right );
   set_r = T(right)->d;
 
-  if( !set_l || !set_r ) /* ({}) & X == ({}) && X & ({}) == ({}) */
+  if( !set_l || !set_l->num_docs || !set_r || !set_r->num_docs )
   {
+    /* ({}) & X == ({}) && X & ({}) == ({}) */
     pop_n_elems(args);
     wf_resultset_push(res);
     return;
   }
 
-  left_size = set_l->num_docs;
-  right_size = set_r->num_docs;
+  left_left = !!(left_size = set_l->num_docs);
+  right_left = !!(right_size = set_r->num_docs);
 
   while( left_left && right_left )
   {
-    if( left_left && left_used ) /* New from left */
+    if( !left_valid ) /* New from left */
     {
-      if( ++lp == left_size )
+      if( ++lp >= left_size )
       {
 	left_left = 0;
 	continue;
@@ -603,15 +599,13 @@ static void f_resultset_intersect( INT32 args )
       {
 	left_doc = set_l->hits[lp].doc_id;
 	left_rank = set_l->hits[lp].ranking;
-	if( !left_rank ) left_rank = right_rank;
-	if( !right_left )right_rank = left_rank;
-	left_used = 0;
+	left_valid = 1;
       }
     }
 
-    if( right_left && right_used ) /* New from right */
+    if( !right_valid ) /* New from right */
     {
-      if( ++rp == right_size )
+      if( ++rp >= right_size )
       {
 	right_left = 0;
 	continue;
@@ -620,33 +614,23 @@ static void f_resultset_intersect( INT32 args )
       {
 	right_doc = set_r->hits[rp].doc_id;
 	right_rank = set_r->hits[rp].ranking;
-	if( !right_rank ) right_rank = left_rank;
-	if( !left_left ) left_rank = right_rank;
-	right_used = 0;
+	right_valid = 1;
       }
     }
 
-    if(!right_left || (left_doc <= right_doc))
-    {
-      if( left_doc == right_doc )
-      {	
-	if(left_doc>last)
-	  wf_resultset_add( res, (last = left_doc),
-			    left_rank+right_rank ); 
-      }
-      left_used=1;
+    if (left_doc < right_doc) {
+      left_valid = 0;
+      continue;
     }
 
-    if(!left_left || (right_doc <= left_doc))
-    {
-      if( right_doc == left_doc )
-      {	
-	if(right_doc>last)
-	  wf_resultset_add( res, (last = right_doc),
-			    left_rank+right_rank );
-      }
-      right_used=1;
+    if (right_doc < left_doc) {
+      right_valid = 0;
+      continue;
     }
+
+    if (right_rank < left_rank) left_rank = right_rank;
+    wf_resultset_add(res, left_doc, left_rank);
+    left_valid = right_valid = 0;
   }
   pop_n_elems( args );
   wf_resultset_push( res );
@@ -867,9 +851,13 @@ static struct object *dup_dateset()
   struct object *o = clone_object( dateset_program, 0 );
   if(THIS->d)
   {
-    ResultSet *d = malloc( THIS->d->num_docs * 8 + 4 );
-    T(o)->d = d;
-    T(o)->allocated_size = T(o)->d->num_docs;
+    ResultSet *d;
+    if (T(o)->allocated_size <= THIS->d->num_docs) {
+      d = xalloc( THIS->d->num_docs * 8 + 4 );
+      if (T(o)->d) free(T(o)->d);
+      T(o)->d = d;
+      T(o)->allocated_size = T(o)->d->num_docs;
+    }
     T(o)->d->num_docs = 0;
   }
   else
