@@ -3,7 +3,6 @@
 #include "global.h"
 #include "stralloc.h"
 #include "global.h"
-RCSID("$Id: 70ad3462e5ea0f8671b73bbe7d774feae8033e46 $");
 #include "pike_macros.h"
 #include "interpret.h"
 #include "program.h"
@@ -20,15 +19,6 @@ RCSID("$Id: 70ad3462e5ea0f8671b73bbe7d774feae8033e46 $");
 #include "blobs.h"
 #include "linkfarm.h"
 
-/* 7.2 compatibility stuff. */
-
-#ifndef PIKE_MODULE_INIT
-/* must be included last */
-#include "module_magic.h"
-#define PIKE_MODULE_INIT void pike_module_init(void)
-#define PIKE_MODULE_EXIT void pike_module_exit(void)
-#endif
-
 struct  tofree
 {
   Blob **blobs;
@@ -78,15 +68,13 @@ static void handle_hit( Blob **blobs,
 			int cutoff )
 {
   int i, j, k, end = 0;
-  Hit *hits = malloc( nblobs * sizeof(Hit) );
+  Hit *hits = calloc( nblobs, sizeof(Hit) );
   unsigned char *nhits = malloc( nblobs );
   unsigned char *pos = malloc( nblobs );
 
   int matrix[65][8];
 
-  MEMSET(matrix, 0, sizeof(matrix) );
-  MEMSET(hits, 0, nblobs * sizeof(Hit) );
-  MEMSET(pos, 0, nblobs );
+  memset(matrix, 0, sizeof(matrix) );
 
   for( i = 0; i<nblobs; i++ )
     nhits[i] = wf_blob_nhits( blobs[i] );
@@ -94,7 +82,7 @@ static void handle_hit( Blob **blobs,
 
   for( i = 0; i<nblobs; i++ )
   {
-    MEMSET( pos, 0, nblobs );
+    memset( pos, 0, nblobs );
     for( j = 0; j<nhits[i]; j++ )
     {
       hits[i] = wf_blob_hit( blobs[i], j );
@@ -213,7 +201,7 @@ static void handle_phrase_hit( Blob **blobs,
   int matrix[65];
   double accum = 0.0;
   
-  MEMSET(matrix, 0, sizeof(matrix) );
+  memset(matrix, 0, sizeof(matrix) );
 
 
   for( i = 0; i<nblobs; i++ )
