@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 07e242b098f0f218217bcd4bfa57286c0e6b7715 $
 */
 
 /*! @module System
@@ -107,13 +106,18 @@ static void memory_shm(INT32 args);
 #define THISOBJ (Pike_fp->current_object)
 #define THIS ((struct memory_storage *)(Pike_fp->current_storage))
 
-static void init_memory(struct object *o)
+static void init_memory(struct object *UNUSED(o))
 {
    THIS->p=NULL;
    THIS->size=0;
    THIS->flags=0;
 }
 
+static void memory__size_object( INT32 UNUSED(args) )
+{
+    push_int(THIS->size);
+}
+
 static void MEMORY_FREE( struct memory_storage *storage )
 {
   if( storage->flags & MEM_FREE_FREE )
@@ -143,7 +147,7 @@ static void MEMORY_FREE( struct memory_storage *storage )
    if (!(STORAGE->p))							\
       Pike_error("%s: no memory in this Memory object\n",FUNC);
 
-static void exit_memory(struct object *o)
+static void exit_memory(struct object *UNUSED(o))
 {
    MEMORY_FREE(THIS);
 }
@@ -162,15 +166,16 @@ static void memory_create(INT32 args)
 {
    if (args)
    {
-      if (sp[-args].type==T_STRING ||
-	  sp[-args].type==T_OBJECT) /* filename to mmap */
+      if (TYPEOF(sp[-args]) == T_STRING ||
+	  TYPEOF(sp[-args]) == T_OBJECT) /* filename to mmap */
 	memory__mmap(args,1,0);
-      else if (sp[-args].type==T_INT && args==1) /* bytes to allocate */
+      else if (TYPEOF(sp[-args]) == T_INT && args==1) /* bytes to allocate */
 	memory_allocate(args);
-      else if(sp[-args].type==T_INT && sp[-args+1].type==T_INT && args==2 )
+      else if(TYPEOF(sp[-args]) == T_INT &&
+	      TYPEOF(sp[-args+1]) == T_INT && args==2 )
 	memory_shm( args );
       else
-	 SIMPLE_BAD_ARG_ERROR("Memory",1,"int or string");
+	 SIMPLE_BAD_ARG_ERROR("create",1,"int|string");
    }
    else
    {
@@ -214,11 +219,11 @@ static void memory_shm( INT32 args )
   MEMORY_FREE(THIS);
 
   if( args < 2 )
-    SIMPLE_TOO_FEW_ARGS_ERROR("Memory.shmat",2);
-  if (Pike_sp[1-args].type!=T_INT )
-    SIMPLE_BAD_ARG_ERROR("Memory.shmat",1,"int(0..)");
-  if (Pike_sp[-args].type!=T_INT )
-    SIMPLE_BAD_ARG_ERROR("Memory.shmat",0,"int(0..)");
+    SIMPLE_TOO_FEW_ARGS_ERROR("shmat",2);
+  if (TYPEOF(Pike_sp[1-args]) != T_INT )
+    SIMPLE_BAD_ARG_ERROR("shmat",1,"int(0..)");
+  if (TYPEOF(Pike_sp[-args]) != T_INT )
+    SIMPLE_BAD_ARG_ERROR("shmat",0,"int(0..)");
 
   if( (id = shmget( Pike_sp[0-args].u.integer,
 		    Pike_sp[1-args].u.integer,
@@ -289,42 +294,42 @@ static void memory__mmap(INT32 args,int complain,int private)
    MEMORY_FREE(THIS); /* we expect this even on error */
 
    if (args<1) 
-      SIMPLE_TOO_FEW_ARGS_ERROR("Memory.mmap",1);
+      SIMPLE_TOO_FEW_ARGS_ERROR("mmap",1);
 
    if (args>=2) {
-      if (sp[1-args].type!=T_INT ||
+      if (TYPEOF(sp[1-args]) != T_INT ||
 	  sp[1-args].u.integer<0)
-	 SIMPLE_BAD_ARG_ERROR("Memory.mmap",2,"int(0..)");
+	 SIMPLE_BAD_ARG_ERROR("mmap",2,"int(0..)");
       else
 	 offset=sp[1-args].u.integer;
    }
 
    if (args>=3) {
-      if (sp[2-args].type!=T_INT ||
+      if (TYPEOF(sp[2-args]) != T_INT ||
 	  sp[2-args].u.integer<0)
-	 SIMPLE_BAD_ARG_ERROR("Memory.mmap",3,"int(0..)");
+	 SIMPLE_BAD_ARG_ERROR("mmap",3,"int(0..)");
       else
 	 size=sp[2-args].u.integer;
    }
 
-   if (sp[-args].type==T_OBJECT)
+   if (TYPEOF(sp[-args]) == T_OBJECT)
    {
       struct object *o=sp[-args].u.object;
       ref_push_object(o);
       push_text("query_fd");
       f_index(2);
-      if (sp[-1].type==T_INT)
-	 SIMPLE_BAD_ARG_ERROR("Memory.mmap",1,
+      if (TYPEOF(sp[-1]) == T_INT)
+	 SIMPLE_BAD_ARG_ERROR("mmap",1,
 			      "(string or) Stdio.File (missing query_fd)");
       f_call_function(1);
-      if (sp[-1].type!=T_INT)
-	 SIMPLE_BAD_ARG_ERROR("Memory.mmap",1,
+      if (TYPEOF(sp[-1]) != T_INT)
+	 SIMPLE_BAD_ARG_ERROR("mmap",1,
 			      "(string or) Stdio.File (weird query_fd)");
       fd=sp[-1].u.integer;
       sp--;
       if (fd<0) {
 	 if (complain)
-	    SIMPLE_BAD_ARG_ERROR("Memory.mmap",1,
+	    SIMPLE_BAD_ARG_ERROR("mmap",1,
 				 "(string or) Stdio.File (file not open)");
 	 else 
 	    RETURN(0);
@@ -334,13 +339,16 @@ static void memory__mmap(INT32 args,int complain,int private)
       osize=file_size(fd);
       THREADS_DISALLOW();
    }
-   else if (sp[-args].type==T_STRING)
+   else if (TYPEOF(sp[-args]) == T_STRING)
    {
       char *filename;
-      get_all_args("Memory.mmap",args,"%s",&filename); /* 8 bit! */
+      get_all_args("mmap",args,"%s",&filename); /* 8 bit! */
       
       THREADS_ALLOW();
       fd = fd_open(filename,fd_RDWR,0);
+      if( fd < 0 )
+          fd = fd_open(filename,fd_RDONLY,0);
+
       if (fd>=0) osize=file_size(fd);
       THREADS_DISALLOW();
 
@@ -444,26 +452,26 @@ static void memory_allocate(INT32 args)
    unsigned char *mem;
 
    if (args>=2)
-      get_all_args("Memory.allocate",args,"%+%+",&size,&c);
+      get_all_args("allocate",args,"%+%+",&size,&c);
    else
-      get_all_args("Memory.allocate",args,"%+",&size);
+      get_all_args("allocate",args,"%+",&size);
 
    /* just to be sure */
    if (size<0)
-      SIMPLE_BAD_ARG_ERROR("Memory.allocate",1,"int(0..)");
+      SIMPLE_BAD_ARG_ERROR("allocate",1,"int(0..)");
    
    pop_n_elems(args);
    if (size>1024*1024) /* threshold */
    {
       THREADS_ALLOW();
-      mem = (unsigned char *)xalloc(size);
-      MEMSET(mem,c,size);
+      mem = xalloc(size);
+      memset(mem,c,size);
       THREADS_DISALLOW();
    }
    else
    {
-      mem = (unsigned char *)xalloc(size);
-      MEMSET(mem,c,size);
+      mem = xalloc(size);
+      memset(mem,c,size);
    }
 
    MEMORY_FREE(THIS);
@@ -527,39 +535,35 @@ static void memory_writeable(INT32 args)
  */
 static void memory_cast(INT32 args)
 {
-   char *s;
-   get_all_args("Memory.cast",args,"%s",&s);
+  struct pike_string *type = Pike_sp[-args].u.string;
+  pop_stack(); /* type have at least one more reference. */
 
-   MEMORY_VALID(THIS,"Memory.cast");
+  MEMORY_VALID(THIS,"Memory.cast");
    
-   if (strncmp(s,"string",5)==0)
-   {
-      pop_n_elems(args);
-      push_string(make_shared_binary_string((char *)THIS->p, THIS->size));
-      return;
-   }
-   if (strncmp(s,"array",5)==0)
-   {
-      struct array *a;
-      size_t i,sz=THIS->size;
-      struct svalue *sv;
+  if (type == literal_string_string)
+  {
+    push_string(make_shared_binary_string((char *)THIS->p, THIS->size));
+  }
+  else if (type == literal_array_string)
+  {
+    struct array *a;
+    size_t i,sz=THIS->size;
+    struct svalue *sv;
 
-      pop_n_elems(args);
+    a=low_allocate_array(sz,0);
 
-      a=low_allocate_array(sz,0);
+    sv=ITEM(a);
+    for (i=0; i<sz; i++)
+    {
+      sv->u.integer=(((unsigned char*)(THIS->p)))[i];
+      sv++;
+    }
+    a->type_field = BIT_INT;
 
-      sv=ITEM(a);
-      for (i=0; i<sz; i++)
-      {
-	 sv->u.integer=(((unsigned char*)(THIS->p)))[i];
-	 sv++;
-      }
-      a->type_field = BIT_INT;
-
-      push_array(a);
-
-      return;
-   }
+    push_array(a);
+  }
+  else
+    push_undefined();
 }
 
 /*! @decl string pread(int(0..) pos,int(0..) len)
@@ -743,7 +747,7 @@ static void pwrite_n(INT32 args,int shift,int reverse,char *func)
 	 case 022: /* 2 -> 2 */
 	    if (reverse)
 	      copy_reverse_string2(d, (unsigned char *)ps->str, ps->len);
-	    else MEMCPY(d,ps->str,ps->len*4);
+	    else memcpy(d,ps->str,ps->len*4);
 	    break;
 	 case 012: /* 1 -> 2 */
 	    if (reverse)
@@ -758,7 +762,7 @@ static void pwrite_n(INT32 args,int shift,int reverse,char *func)
 	 case 011: /* 1 -> 1 */
 	    if (reverse)
 	      copy_reverse_string1(d, (unsigned char *)ps->str,ps->len);
-	    else MEMCPY(d,ps->str,ps->len*2);
+	    else memcpy(d,ps->str,ps->len*2);
 	    break;
 	 case 001: /* 0 -> 1 */
 	    if (reverse)
@@ -766,7 +770,7 @@ static void pwrite_n(INT32 args,int shift,int reverse,char *func)
 	    else convert_0_to_1((p_wchar1*)d, STR0(ps), ps->len);
 	    break;
 	 case 000:
-	    MEMCPY(d,ps->str,ps->len);
+	    memcpy(d,ps->str,ps->len);
 	    break;
 	 default:
 	    Pike_error("Illegal state %d -> %d\n",ps->size_shift,shift);
@@ -808,7 +812,7 @@ static void memory_index(INT32 args)
    {
       INT_TYPE pos;
       size_t rpos = 0;
-      get_all_args("Memory.`[]",args,"%i",&pos);
+      get_all_args("`[]",args,"%i",&pos);
       if (pos<0) {
 	 if ((off_t)-pos>=DO_NOT_WARN((off_t)THIS->size))
 	    Pike_error("Memory.`[]: Index is out of range\n");
@@ -834,7 +838,7 @@ static void memory_index(INT32 args)
 	 INT_TYPE pos1,pos2;
 	 size_t rpos1,rpos2;
 
-	 get_all_args("Memory.`[]",args,"%i%i",&pos1,&pos2);
+	 get_all_args("`[]",args,"%i%i",&pos1,&pos2);
 	 if (pos1<0) rpos1=0; else rpos1=(size_t)pos1;
 	 if ((size_t)pos2>=THIS->size) rpos2=THIS->size-1; 
 	 else rpos2=(size_t)pos2;
@@ -863,7 +867,7 @@ static void memory_index_write(INT32 args)
    {
       INT_TYPE pos,ch;
       size_t rpos = 0;
-      get_all_args("Memory.`[]=",args,"%i%i",&pos,&ch);
+      get_all_args("`[]=",args,"%i%i",&pos,&ch);
       if (pos<0) 
 	 if ((off_t)-pos>=DO_NOT_WARN((off_t)THIS->size))
 	    Pike_error("Memory.`[]=: Index is out of range\n");
@@ -886,7 +890,7 @@ static void memory_index_write(INT32 args)
      INT_TYPE pos1, pos2;
      struct pike_string *ps;
 
-     get_all_args("Memory.`[]=", args, "%i%i%S", &pos1, &pos2, &ps);
+     get_all_args("`[]=", args, "%i%i%S", &pos1, &pos2, &ps);
 
      if (pos1 < 0) pos1 = 0;
      if (pos2 < 0) pos2 = 0;
@@ -896,7 +900,7 @@ static void memory_index_write(INT32 args)
 		  "not equally long (%ld v/s %ld; can't resize memory)\n",
 		  DO_NOT_WARN((long)ps->len), (long)pos2-(long)pos1);
      else
-       MEMCPY(THIS->p+pos1, ps->str, ps->len);
+       memcpy(THIS->p+pos1, ps->str, ps->len);
 
      ref_push_string(ps);
    }
@@ -942,6 +946,9 @@ void init_system_memory(void)
 
    ADD_FUNCTION("allocate",memory_allocate,
 		tFunc(tIntPos tOr(tByte,tVoid),tVoid),0);
+
+   ADD_FUNCTION("_size_object",memory__size_object,
+                tFunc(tVoid,tInt),0);
    
    ADD_FUNCTION("free",memory_free,tFunc(tVoid,tVoid),0);
 
@@ -950,7 +957,7 @@ void init_system_memory(void)
 
    ADD_FUNCTION("_sizeof",memory__sizeof,tFunc(tVoid,tIntPos),0);
    ADD_FUNCTION("cast",memory_cast,
-		tFunc(tStr,tOr(tArr(tInt),tStr)),0);
+		tFunc(tStr,tOr(tArr(tInt),tStr)),ID_PRIVATE);
 
    ADD_FUNCTION("`[]",memory_index,
 		tOr(tFunc(tInt,tInt),
