@@ -3270,7 +3270,7 @@ void init_object(void)
    *     opcode F_MAGIC_INDEX.
    */
   ADD_FUNCTION("`()", f_magic_index,
-	       tFunc(tStr tOr3(tVoid,tObj,tDeprecated(tInt)) tOr(tVoid,tInt),
+	       tFunc(tStr tOr3(tVoid,tObj,tInt) tOr(tVoid,tInt),
 		     tMix), ID_PROTECTED);
   magic_index_program=end_program();
 
@@ -3283,8 +3283,8 @@ void init_object(void)
    *     opcode F_MAGIC_SET_INDEX.
    */
   ADD_FUNCTION("`()", f_magic_set_index,
-	       tFunc(tStr tMix tOr3(tVoid,tObj,tDeprecated(tInt))
-		     tOr(tVoid,tInt), tVoid), ID_PROTECTED);
+	       tFunc(tStr tMix tOr3(tVoid,tObj,tInt)
+		     tOr(tVoid,tInt), tZero), ID_PROTECTED);
   magic_set_index_program=end_program();
 
   /* ::_indices() */
@@ -3296,7 +3296,7 @@ void init_object(void)
    *     opcode F_MAGIC_INDICES.
    */
   ADD_FUNCTION("`()", f_magic_indices,
-	       tFunc(tOr3(tVoid,tObj,tDeprecated(tInt)) tOr(tVoid,tInt),
+	       tFunc(tOr3(tVoid,tObj,tInt) tOr(tVoid,tInt),
 		     tArr(tStr)), ID_PROTECTED);
   magic_indices_program=end_program();
 
@@ -3309,7 +3309,7 @@ void init_object(void)
    *     opcode F_MAGIC_VALUES.
    */
   ADD_FUNCTION("`()", f_magic_values,
-	       tFunc(tOr3(tVoid,tObj,tDeprecated(tInt)) tOr(tVoid,tInt),
+	       tFunc(tOr3(tVoid,tObj,tInt) tOr(tVoid,tInt),
 		     tArray), ID_PROTECTED);
   magic_values_program=end_program();
 
@@ -3322,7 +3322,7 @@ void init_object(void)
    *     opcode F_MAGIC_TYPES.
    */
   ADD_FUNCTION("`()", f_magic_types,
-	       tFunc(tOr3(tVoid,tObj,tDeprecated(tInt)) tOr(tVoid,tInt),
+	       tFunc(tOr3(tVoid,tObj,tInt) tOr(tVoid,tInt),
 		     tArr(tType(tMix))), ID_PROTECTED);
   magic_types_program=end_program();
 
