@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 80428396bde247a5de21bd0e512909ff4abd10f2 $
 */
 
 #ifndef PIKE_ERROR_H
@@ -52,13 +51,16 @@ static inline void Pike_fatal (const char *fmt, ...)
 #if 1
 PMOD_EXPORT extern const char msg_fatal_error[];
 #define Pike_fatal \
- (fprintf(stderr,msg_fatal_error,__FILE__,__LINE__),debug_fatal)
+ (fprintf(stderr,msg_fatal_error,__FILE__,(long)__LINE__),debug_fatal)
 #else
 /* This is useful when debugging assembler code sometimes... -Hubbe */
 #define Pike_fatal \
  (fprintf(stderr,"%s: Fatal error:\n",__FILE__ ":" DEFINETOSTR(__LINE__) ),debug_fatal)
 #endif
 
+#define pike_fatal_dloc							\
+ (fprintf (stderr, msg_fatal_error, DLOC_ARGS), debug_fatal)
+
 #ifndef NO_PIKE_SHORTHAND
 #define fatal Pike_fatal
 #endif
@@ -121,7 +123,7 @@ typedef struct JMP_BUF
 } JMP_BUF;
 
 PMOD_EXPORT extern struct svalue throw_value;
-extern int throw_severity;
+PMOD_EXPORT extern int throw_severity;
 
 #ifdef PIKE_DEBUG
 PMOD_EXPORT extern const char msg_unsetjmp_nosync_1[];
@@ -251,7 +253,6 @@ PMOD_EXPORT JMP_BUF *init_recovery(JMP_BUF *r, size_t stack_pop_levels DEBUG_INI
 PMOD_EXPORT DECLSPEC(noreturn) void pike_throw(void) ATTRIBUTE((noreturn));
 PMOD_EXPORT void push_error(const char *description);
 PMOD_EXPORT DECLSPEC(noreturn) void low_error(const char *buf) ATTRIBUTE((noreturn));
-PMOD_EXPORT void Pike_vsnprintf(char *str, size_t size, const char *fmt, va_list ap);
 PMOD_EXPORT void va_make_error (const char *fmt, va_list args);
 PMOD_EXPORT void DECLSPEC(noreturn) va_error(const char *fmt, va_list args) ATTRIBUTE((noreturn));
 PMOD_EXPORT void make_error (const char *fmt, ...);
