@@ -2,7 +2,6 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: 3e408d76742e3c5690b6bfed2c5b49e551fc91ea $
 */
 
 #include "global.h"
@@ -56,9 +55,7 @@ static struct program *creds_program;
 
 static int valid_creds_object(struct object *o)
 {
-  return o &&
-    get_storage(o, creds_program) == o->storage &&
-    OBJ2CREDS(o)->user;
+  return o && get_storage(o, creds_program) == o->storage && OBJ2CREDS(o)->user;
 }
 
 static void restore_creds(struct object *creds)
@@ -115,7 +112,7 @@ static void f_call_with_creds(INT32 args)
       if(!CHECK_SECURITY(SECURITY_BIT_SECURITY) &&
 	 !(Pike_fp->current_object->prot && 
 	   (OBJ2CREDS(Pike_fp->current_object->prot)->may_always & SECURITY_BIT_SECURITY)))
-	Pike_error("call_with_creds: permission denied.\n");
+	Pike_error("Permission denied.\n");
       
       break;
 
@@ -124,7 +121,7 @@ static void f_call_with_creds(INT32 args)
   }
     
   if(!valid_creds_object(o))
-    Pike_error("call_with_creds: Not a valid creds object.\n");
+    Pike_error("Not a valid creds object.\n");
 
   if(CURRENT_CREDS) add_ref(CURRENT_CREDS);
 
@@ -306,7 +303,7 @@ static void creds_apply(INT32 args)
       free_object(Pike_sp[-args].u.array->prot);
     add_ref( Pike_sp[-args].u.array->prot=Pike_fp->current_object );
   }else{
-    Pike_error("creds->apply(): permission denied.\n");
+    Pike_error("Permission denied.\n");
   }
   pop_n_elems(args);
 }
