@@ -1,7 +1,8 @@
 #include "stack_allocator.h"
 
+MALLOC_FUNCTION
 static struct chunk * alloc_chunk(size_t size) {
-    struct chunk * c = (struct chunk*)xalloc(sizeof(struct chunk) + size);
+    struct chunk * c = xalloc(sizeof(struct chunk) + size);
     c->data = c->top = c + 1;
     c->size = size;
     c->prev = NULL;
@@ -22,7 +23,9 @@ void stack_alloc_enlarge(struct stack_allocator * a, size_t len) {
         len |= len >> 4;
         len |= len >> 8;
         len |= len >> 16;
+#if SIZEOF_INT_TYPE > 4
         len |= len >> 32;
+#endif
         len ++;
     }
     a->cur = alloc_chunk(len);
@@ -40,15 +43,3 @@ void stack_alloc_destroy(struct stack_allocator * a) {
 
     a->cur = NULL;
 }
-
-size_t stack_alloc_count(struct stack_allocator * a) {
-    struct chunk * c = a->cur;
-    size_t size = 0;
-
-    while (c) {
-        size += c->size;
-        c = c->prev;
-    }
-
-    return size;
-}
